"""
  New job class to run Prod5 Divergent simulations
  Only Paranal in the Alpha layout is supported, using zstd compression
  for simtel output files
          FDP, May 2022

  Adapted from Prod5MCPipeNSBJob.py by JB June 2020 (having a look at what was done for 
  Prod5bMCPipeAlphaSSTsFullMoonNSBJob.py by OG, January 2022)
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
from CTADIRAC.Interfaces.API.Prod5MCPipeNSBJob import Prod5MCPipeNSBJob


class Prod5bMCPipeAlphaDivergentJob(Prod5MCPipeNSBJob):
  """ Job extension class for Prod5 MC Divergent simulations,
  takes care of 

  """

  def __init__(self, cpu_time=259200):
    """ Constructor takes almosst everything from base class

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Prod5MCPipeNSBJob.__init__(self, cpu_time)
    self.setCPUTime(cpu_time)
    self.setName('Prod5MC_Generation')
    self.version = '2020-06-29b'
    self.compiler = 'gcc83_matchcpu'
    self.configuration_id = 14
    self.n_shower = 100
    self.output_pattern = './Data/*.simtel.zst'
    self.base_path = '/vo.cta.in2p3.fr/MC/PROD5/'
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])
    self.metadata = collections.OrderedDict()

  def set_meta_data(self):
    """ define the common meta data of the application
    """
    # The order of the metadata dictionary is important,
    # since it's used to build the directory structure
    self.metadata['array_layout'] = 'Alpha'
    self.metadata['site'] = self.cta_site
    self.metadata['particle'] = self.particle
    # for air shower simulation means North=0 and South=180
    # but here piped into tel_sim so North=180 and South=0
    if self.pointing_dir == 'North':
      self.metadata['phiP'] = 180
    if self.pointing_dir == 'South':
      self.metadata['phiP'] = 0
    self.metadata['thetaP'] = float(self.zenith_angle)
    # back to common write up from class
    self.metadata[self.program_category + '_prog'] = self.prog_name
    self.metadata[self.program_category + '_prog_version'] = self.version
    self.metadata['data_level'] = self.output_data_level
    self.metadata['configuration_id'] = self.configuration_id

  def setupWorkflow(self, debug=False):
    """ Override the base class job workflow to adapt to divergent simulations
        All parameters shall have been defined before that method is called.
    """
    # step 1 - debug only
    i_step = 1
    if debug:
      ls_step = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
      ls_step['Value']['name'] = 'Step%i_LS_Init' % i_step
      ls_step['Value']['descr_short'] = 'list files in working directory'

      env_step = self.setExecutable('/bin/env', logFile='Env_Log.txt')
      env_step['Value']['name'] = 'Step%i_Env' % i_step
      env_step['Value']['descr_short'] = 'Dump environment'
      i_step += 1

    # step 2
    sw_step = self.setExecutable('cta-prod-setup-software',
                                 arguments='-p %s -v %s -a simulations -g %s' %
                                 (self.package, self.version, self.compiler),
                                 logFile='SetupSoftware_Log.txt')
    sw_step['Value']['name'] = 'Step%i_SetupSoftware' % i_step
    sw_step['Value']['descr_short'] = 'Setup software'
    i_step += 1

    # step 3 running
    prod_script = './dirac_prod5_alpha_divergent_run'
    if 'gcc10' in self.compiler:
      prod_exe = './dirac_singularity_run'
      prod_args = '%s --start_run %s --run %s %s %s %s %s' % \
                  (prod_script, self.start_run_number, self.run_number,
                   self.cta_site, self.particle, self.pointing_dir,
                   self.zenith_angle)
    else:
      prod_exe = prod_script
      prod_args = '--start_run %s --run %s %s %s %s %s' % \
                  (self.start_run_number, self.run_number,
                   self.cta_site, self.particle, self.pointing_dir,
                   self.zenith_angle)

    cs_step = self.setExecutable(prod_exe, arguments=prod_args,
                                 logFile='CorsikaSimtel_Log.txt')
    cs_step['Value']['name'] = 'Step%i_CorsikaSimtel' % i_step
    cs_step['Value']['descr_short'] = 'Run Corsika piped into simtel'
    i_step += 1

    # step 4 - debug only
    if debug:
      ls_step = self.setExecutable('/bin/ls -Ralhtr', logFile='LS_End_Log.txt')
      ls_step['Value']['name'] = 'Step%i_LS_End' % i_step
      ls_step['Value']['descr_short'] = 'list files in working directory and sub-directory'
      i_step += 1
    
    # step 5  verify the number of events in the simtel file and define meta data, upload file on SE and register in catalogs
    self.set_meta_data()
    md_json = json.dumps(self.metadata)
    meta_data_field = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                       'particle': 'VARCHAR(128)',
                       'phiP': 'float', 'thetaP': 'float',
                       self.program_category + '_prog': 'VARCHAR(128)',
                       self.program_category + '_prog_version': 'VARCHAR(128)',
                       'data_level': 'int', 'configuration_id': 'int', 'merged': 'int'}
    md_field_json = json.dumps(meta_data_field)
    
    i_substep = 0
    for div_ang in ['0.0022', '0.0043', '0.008', '0.01135', '0.01453']:
      
      data_output_pattern = 'Data/*-div%s*.simtel.zst'%(div_ang)
     
      #verify the number of events in the simtel file
      mgv_step = self.setExecutable('dirac_simtel_check',
                                  arguments="'%s'" %
                                  (data_output_pattern),
                                  logFile='Verify_n_showers_div%s_Log.txt'%(div_ang))
      mgv_step['Value']['name'] = 'Step%s.%s_VerifyNShowers' % (i_step,i_substep)
      mgv_step['Value']['descr_short'] = 'Verify number of showers'
      
      #define meta data, upload file on SE and register in catalogs

      # Upload and register data 
      file_meta_data = {'runNumber': self.run_number, 'nsb': 1, 'div_ang': div_ang} #adding a new meta data field 
      file_md_json = json.dumps(file_meta_data)

      dm_step = self.setExecutable('cta-prod-managedata',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                 (md_json, md_field_json, file_md_json,
                                  self.base_path, data_output_pattern, self.package,
                                  self.program_category, self.catalogs),
                                 logFile="DataManagement_div%s_Log.txt"%(div_ang))
      dm_step['Value']['name'] = 'Step%s.%s_DataManagement' % (i_step,i_substep)
      dm_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
      
      # Upload and register log_and_hist file
      log_file_pattern = 'Data/*-div%s*.log_hist.tar'%(div_ang)
      log_step = self.setExecutable('cta-prod-managedata',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                  (md_json, md_field_json, file_md_json,
                                   self.base_path, log_file_pattern, self.package,
                                   self.program_category, self.catalogs),
                                  logFile="LogManagement_div%s_Log.txt"%(div_ang))
      log_step['Value']['name'] = 'Step%s.%s_LogManagement' % (i_step,i_substep)
      log_step['Value']['descr_short'] = 'Save log to SE and register them in DFC'
      
      i_substep += 1
    i_step += 1

    # Step 6 - debug only
    if debug:
      ls_step = self.setExecutable('/bin/ls -Ralhtr', logFile='LS_End_Log.txt')
      ls_step['Value']['name'] = 'Step%s_LSHOME_End' % i_step
      ls_step['Value']['descr_short'] = 'list files in Home directory'
      #i_step += 1

    # Number of showers is passed via an environment variable
    self.setExecutionEnv({'NSHOW': '%s' % self.n_shower})



import DIRAC
from DIRAC.Core.Base import Script

Script.setUsageMessage('\n'.join(['Get storage elements usage of production SEs',
                                  'Usage:',
                                  '%s <file with output of lcg-infosites --vo vo.cta.in2p3.fr se>' % Script.scriptName,
                                  '\ne.g: %s ccsrm02.in2p3.fr (default is all SEs)' % Script.scriptName
                                  ]))


Script.parseCommandLine(ignoreErrors=True)

from DIRAC.Core.Utilities.PrettyPrint import printTable
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations

def main():
  args = Script.getPositionalArgs()

  if len(args) > 2:
    Script.showHelp()

  # Default list
  if len(args) == 1:
    opsHelper = Operations()
    SEList = opsHelper.getValue('ProductionSEs/Hosts', [])
  else:
    SEList = args[:2]

  sedict = {}
  for SE in SEList:
    sedict[SE] = []

  fields = ['SE', 'Available(TB)', 'Used(TB)', 'Total(TB)', 'Used(%)']
  records = []

  fp = open(args[0], "rt")

  for se in fp:
    if len(se.split()) == 4:
      spacedict = {}
      SE = se.split()[3]
      if SE in SEList and se.split()[0] != 'n.a' and se.split()[1] != 'n.a':
        # ## convert into TB
        available = float(se.split()[0]) / 1e9
        used = float(se.split()[1]) / 1e9
        spacedict['Available'] = available
        spacedict['Used'] = used
        spacedict['Total'] = available + used
        sedict[SE].append(spacedict)

  for SE in SEList:
    for spacedict in sedict[SE]:
      available = '%.1f' % spacedict['Available']
      used = '%.1f' % spacedict['Used']
      total = '%.1f' % spacedict['Total']
      fraction_used = spacedict['Used'] / spacedict['Total'] * 100
      fraction_used = '%.1f' % fraction_used
      records.append([SE, available, used, total, fraction_used])

  printTable(fields, records)
  DIRAC.exit()

if __name__ == '__main__':
  main()


#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
from multiprocessing import Pool
import signal
import os

# DIRAC imports
from DIRAC import gLogger
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

Script.setUsageMessage("""
Bulk get replicas from a list of lfns
Usage:
   cta-prod-get-file [options] <ascii file with lfn list>
""")

Script.parseCommandLine(ignoreErrors=True)
# Import of DIRAC API must comes after parseCommandLine
from DIRAC.Interfaces.API.Dirac import Dirac

TEMPDIR = '.incomplete'

def sigint_handler(signum, frame):
  '''
  Raise KeyboardInterrupt on SIGINT (CTRL + C)
  This should be the default, but apparently Dirac changes it.
  '''
  raise KeyboardInterrupt()

def getfile(lfn):
  dirac = Dirac()
  gLogger.notice('Start downloading file', lfn)
  res = dirac.getFile(lfn, destDir=TEMPDIR)

  if not res['OK']:
    gLogger.error('Error downloading file', lfn)
    return res['Message']

  name = os.path.basename(lfn)
  os.rename(os.path.join('.incomplete', name), name)
  gLogger.notice('Successfully downloaded file', lfn)

@Script()
def main():
  args = Script.getPositionalArgs()
  if len(args) == 1:
    infile = args[0]
  else:
    Script.showHelp()

  # put files currently downloading in a subdirectory
  # to know which files have already finished
  if not os.path.exists(TEMPDIR):
    os.makedirs(TEMPDIR)

  infileList = read_inputs_from_file(infile)
  # see https://stackoverflow.com/a/35134329/3838691
  # ignore sigint before creating the pool,
  # so child processes inherit the setting, we will terminate them
  # if the master process catches sigint
  signal.signal(signal.SIGINT, signal.SIG_IGN)
  p = Pool(10)
  # add the original handler back
  signal.signal(signal.SIGINT, sigint_handler)

  try:
    future = p.map_async(getfile, infileList)

    while not future.ready():
      future.wait(5)
  except (SystemExit, KeyboardInterrupt):
    gLogger.fatal('Received SIGINT, waiting for subprocesses to terminate')
    p.close()
    p.terminate()
  finally:
    p.close()
    p.join()
    if len(os.listdir(TEMPDIR)) == 0:
      os.rmdir(TEMPDIR)

if __name__ == '__main__':
  main()


#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
from multiprocessing import Pool

# DIRAC imports
from DIRAC import gLogger
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

Script.setUsageMessage("""
Extension of dirac-dms-replicate-lfn in multi-thread mode
Allows for bulk replication of a list of LFNs to a destination Storage Element
Usage:
   cta-prod-replicate-lfn <ascii file with lfn list> <SE>
""")

Script.parseCommandLine(ignoreErrors=True)

args = Script.getPositionalArgs()
if len(args) > 1:
  infile = args[0]
  SE = args[1]
else:
  Script.showHelp()

# Import of Dirac must comes after parseCommandLine
from DIRAC.Interfaces.API.Dirac import Dirac

@Script()
def main():
  infileList = read_inputs_from_file(infile)
  p = Pool(10)
  p.map(replicateFile, infileList)

def replicateFile(lfn):
  dirac = Dirac()
  res = dirac.replicateFile(lfn, SE)
  if not res['OK']:
    gLogger.error('Error replicating file', lfn)
    return res['Message']

if __name__ == '__main__':
  main()



#!/usr/bin/env python

__RCSID__ = "$Id$"


# DIRAC imports
import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

Script.setUsageMessage("""
Check replica location against the a given SE
Usage:
   cta-check-get-replicas <ascii file with lfn list> <destSE>
""")

Script.parseCommandLine(ignoreErrors=True)

from DIRAC.Resources.Catalog.FileCatalog import FileCatalog
fc = FileCatalog()

args = Script.getPositionalArgs()
if len(args) > 0:
  infile = args[0]
  destSE = args[1]
else:
  Script.showHelp()

f0 = open("%s_NoReplicas.lfns" % (infile), "w")
f1 = open("%s_SingleReplica_%s.lfns" % (infile, destSE), "w")
f2 = open("%s_MultipleReplicas_%s.lfns" % (infile, destSE), "w")
f3 = open("%s_MissingReplica_%s.lfns" % (infile, destSE), "w")

@Script()
def main():
  resList = read_inputs(infile)
  for (transID, lfn) in resList:
    checkReplicas(transID, lfn)
  gLogger.notice('Files dumped: check %s_*.lfns' % (infile))

def read_inputs(file_path):
  content = open(file_path, 'rt').readlines()
  resList = []
  for line in content:
    transID = line.strip().split(" ")[0]
    lfn = line.strip().split(" ")[1]
    if line != "\n":
      resList.append((transID, lfn))
  return resList

def checkReplicas(transID, lfn):
  res = fc.getReplicas(lfn)
  if not res['OK']:
    gLogger.notice('Failed to get replicas for lfn', lfn)
    return res['Message']
  if lfn in res['Value']['Successful']:
    resDict = res['Value']['Successful'][lfn]
    nReplicas = len(resDict)
    if destSE in resDict:
      if nReplicas == 1:
        f1.write("%s %s\n" % (transID, lfn))
      else:
        f2.write("%s %s\n" % (transID, lfn))
    else:
      if nReplicas == 0:
        f0.write("%s %s\n" % (transID, lfn))
      else:
        f3.write("%s %s\n" % (transID, lfn))
  else:
    res = DIRAC.S_ERROR('Failed to get replicas for lfn %s' % lfn)
    gLogger.notice(res['Message'])
    return res

if __name__ == '__main__':
  main()



#!/usr/bin/env python
""" Collection of functions to download data files
matching a given pattern / query.

         cta-prod3-get-matching-data.py JB, LA 2016
"""

__RCSID__ = "$Id$"

# generic imports
import os
import six

# DIRAC import Script
import DIRAC
from DIRAC.Core.Base import Script

Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s funcName' % Script.scriptName,
                                  'Arguments:',
                                  '  stepName: sub5, sub2sub5, HB9SCT, HB9merged',
                                  '\ne.g: %s sub5' % Script.scriptName
                                  ]))

Script.parseCommandLine()

# Other DIRAC imports
from DIRAC.Interfaces.API.Dirac import Dirac
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient


def downloadFile(lfn):
  """ Download a file using DMS
  Keyword arguments:
  lfn -- a logical file name
  """
  # First check if the file exists
  fc = FileCatalogClient()
  res = fc.exists(lfn)
  if not res['OK']:
    DIRAC.gLogger.error(res['Message'])

  if res['Value']['Successful'][lfn]:
    DIRAC.gLogger.info('Downloading ', lfn)
    dm = DataManager()
    res = dm.getFile(lfn)
    if not res['OK']:
      DIRAC.gLogger.error(res['Message'])

    if lfn not in res['Value']['Successful']:
      DIRAC.gLogger.error('Could not download %s' % lfn)
      return DIRAC.S_ERROR()
  else:
    return DIRAC.S_ERROR('File not found in Catalog')

  return DIRAC.S_OK()


def getSub5():
  """ Download subarray-5 files corresponding to a list of subarray-2 files

  Keyword arguments:
  none -- none
  """
  DIRAC.gLogger.info('Get Subarray-5 files')
  # get JDL
  dirac = Dirac()
  resJDL = dirac.getJobJDL(os.environ['JOBID'])

  # get list of output files
  idata = resJDL['Value']['InputData']

  # dowload files
  for sub2 in idata:
    DIRAC.gLogger.debug("Input %s " % sub2)
    sub5 = sub2.strip('\n').replace('subarray-2-nosct', 'subarray-5-nosct')
    downloadFile(sub5)

  return DIRAC.S_OK()


def getSub2Sub5():
  """ Download subarray-2 and 5 files corresponding to a list of subarray-1 files

  Keyword arguments:
  none -- none
  """
  DIRAC.gLogger.info('Get Subarray-2 and 5 files')
  # get JDL
  dirac = Dirac()
  resJDL = dirac.getJobJDL(os.environ['JOBID'])

  # get list of output files
  idata = resJDL['Value']['InputData']

  # dowload files
  for sub1 in idata:
    DIRAC.gLogger.debug("Input %s " % sub1)
    sub2 = sub1.strip('\n').replace('subarray-1-nosct', 'subarray-2-nosct')
    downloadFile(sub2)
    sub5 = sub1.strip('\n').replace('subarray-1-nosct', 'subarray-5-nosct')
    downloadFile(sub5)

  return DIRAC.S_OK()


def getHB9SCT():
  """ Download SCT files corresponding to a list of HB9 merged files

  Keyword arguments:
  none -- none
  """
  DIRAC.gLogger.info('Get HB9-SCT files')
  # get JDL
  dirac = Dirac()
  resJDL = dirac.getJobJDL(os.environ['JOBID'])
  # get list of output files
  idata = resJDL['Value']['InputData']

  # dowload files
  for merged in idata:
    DIRAC.gLogger.debug("Input %s " % merged)
    sct = merged.strip('\n').replace('merged', 'SCT').replace('cta-prod3', 'cta-prod3-sct')
    res = downloadFile(sct)

  return res


def getHB9merged():
  """ Download HB9 merged files corresponding to a list of SCT files

  Keyword arguments:
  none -- none
  """
  DIRAC.gLogger.info('Get HB9 files')
  # get JDL
  dirac = Dirac()
  resJDL = dirac.getJobJDL(os.environ['JOBID'])

  # get list of output files
  idata = resJDL['Value']['InputData']
  if isinstance(idata, six.string_types):
    idata = []
    if 'LFN' in resJDL['Value']['InputData']:
      idata.append(resJDL['Value']['InputData'].split('LFN:')[1])
    else:
      idata.append(resJDL['Value']['InputData'])
  else:
    idata = resJDL['Value']['InputData']

  # dowload files
  count = 0
  for sct in idata:
    DIRAC.gLogger.notice("Input %s " % sct)
    merged = sct.strip('\n').replace('SCT', 'merged').replace('cta-prod3-sct', 'cta-prod3')
    res = downloadFile(merged)
    if res['OK']:
      count = +1

  if count == 0:
    return DIRAC.S_ERROR("No files downloaded")

  return DIRAC.S_OK()

# Main
def main():
  """ simple wrapper to download data files matching a pattern

  Keyword arguments:
  args -- a list of arguments in order []
  """
  # check command line
  args = Script.getPositionalArgs()
  if len(args) != 1:
    res = DIRAC.S_ERROR()
    res['Message'] = 'Just give the function you wish to use: sub5, sub2sub5, HB9SCT, HB9merged'
    return res

  # now do something
  funcName = args[0]

  # What shall we verify ?
  if funcName == "sub5":
    res = getSub5()
  elif funcName == "sub2sub5":
    res = getSub2Sub5()
  elif funcName == "HB9SCT":
    res = getHB9SCT()
  elif funcName == "HB9merged":
    res = getHB9merged()
  else:
    res = DIRAC.S_ERROR()
    res['Message'] = 'Function "%s" not known' % funcName

  if not res['OK']:
    DIRAC.gLogger.error ( res['Message'] )
    DIRAC.exit(-1)

  DIRAC.exit()


####################################################
if __name__ == '__main__':
  main()


#!/usr/bin/env python
"""
Add a dataset from a query specified in a json file, e.g.:

{"MCCampaign": "PROD5b",
"particle": "proton",
"site": "Paranal"}

The created dataset will get the name of the json file (without the extension)

Usage:
   cta-prod-add-dataset <json file with dataset query dict>
Arguments:
   json file with dataset query dict
   The file should have the .json extension, e.g. my_dataset.json
"""

__RCSID__ = "$Id$"

import json
import ast

import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC import gLogger
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

@Script()
def main():
  fc = FileCatalogClient()
  Script.parseCommandLine(ignoreErrors=True)
  argss = Script.getPositionalArgs()
  if len(argss) == 1:
    inputJson = argss[0]
    if not inputJson.endswith(".json"):
      Script.showHelp()
  else:
    Script.showHelp()

  # read the meta data query from json file
  content = open(inputJson, 'rt').readlines()
  for line in content:
    json_string = json.dumps(ast.literal_eval(line))

  MDdict = json.loads(json_string)

  # ensure the integer values are written in the right format
  for key, value in MDdict.items():
    if isinstance(value, float):
      MDdict[key] = {'=': value}

  datasetName = inputJson.split('.json')[0]
  datasetPath = '/' + datasetName
  res = (fc.addDataset({datasetPath: MDdict}))

  if not res['OK']:
    gLogger.error("Failed to add dataset %s: %s" % (datasetName, res['Message']))
    DIRAC.exit(-1)

  if datasetName in res['Value']['Failed']:
    gLogger.error("Failed to add dataset %s: %s" % (datasetName, res['Value']['Failed'][datasetName]))
    DIRAC.exit(-1)

  gLogger.notice("Successfully added dataset", datasetName)
  DIRAC.exit()

####################################################
if __name__ == '__main__':
  main()



""" Launcher script to launch a Prod3 EvnDispProd3SCTSingJob
on the WMS or create a Transformation.

    https://forge.in2p3.fr/issues/441709

   November 16th 2020 - J. Bregeon
                  bregeon@in2p3.fr
"""

from copy import copy

from DIRAC.Core.Base import Script
Script.setUsageMessage(
    '\n'.join(
        [
            __doc__.split('\n')[1],
            'Usage:',
            '  %s mode file_path (trans_name) group_size' %
            Script.scriptName,
            'Arguments:',
            '  mode: WMS for testing, TS for production',
            '  trans_name: name of the transformation',
            '  input_dataset_name: name of the input dataset',
            '  group_size: n files to process',
            '\ne.g: python %s.py WMS MyNewTrans Prod4-Paranal-gamma-North-DL-3 5' %
            Script.scriptName,
        ]))

Script.parseCommandLine()

import DIRAC
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from CTADIRAC.Interfaces.API.EvnDispProd3SCTSingJob import EvnDispProd3SCTSingJob
from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC.Interfaces.API.Dirac import Dirac
from CTADIRAC.Core.Utilities.tool_box import get_dataset_MQ


def submit_trans(job, trans_name, input_meta_query, group_size):
  """ Create a transformation executing the job workflow
  """
  DIRAC.gLogger.notice('submit_trans : %s' % trans_name)

  # Initialize JOB_ID
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))

  trans = Transformation()
  trans.setTransformationName(trans_name)  # this must be unique
  trans.setType("DataReprocessing")
  trans.setDescription("Prod3SCT EventDisplay TS")
  trans.setLongDescription("Prod3SCT EventDisplay processing")  # mandatory
  trans.setBody(job.workflow.toXML())
  trans.setGroupSize(group_size)
  trans.setInputMetaQuery(input_meta_query)
  result = trans.addTransformation()  # transformation is created here
  if not result['OK']:
    return result
  trans.setStatus("Active")
  trans.setAgentType("Automatic")
  trans_id = trans.getTransformationID()
  return trans_id


def submit_wms(job):
  """ Submit the job to the WMS
  @todo launch job locally
  """
  dirac = Dirac()
  base_path = '/vo.cta.in2p3.fr/MC/PROD3/scratch/Paranal/gamma/simtel/954/Data/000xxx'
  # input_data = ['%s/gamma_20deg_0deg_run107___cta-prod3_desert-2150m-Paranal-merged.simtel.gz' % base_path]  # CNAF
  input_data = ['%s/gamma_20deg_0deg_run312___cta-prod3-sct_desert-2150m-Paranal-SCT.simtel.gz' % base_path]  # CC-IN2P3

  job.setInputData(input_data)
  job.setJobGroup('EvnDispProd5')
  result = dirac.submitJob(job)
  if result['OK']:
    Script.gLogger.notice('Submitted job:', result['Value'])
  return result


def launch_job(args):
  """ Simple launcher to instanciate a Job and setup parameters
      from positional arguments given on the command line.

      Parameters:
      args -- mode (trans_name dataset_name group_size)
  """
  DIRAC.gLogger.notice('Running EventDisplay jobs')
  # get arguments
  mode = args[0]

  if mode == 'TS':
    trans_name = args[1]
    dataset_name = args[2]
    group_size = int(args[3])

  # job setup - 72 hours
  job = EvnDispProd3SCTSingJob(cpuTime=259200.)
  # override for testing
  job.setName('Prod3SCT_EvnDisp')
  # output
  job.setOutputSandbox(['*Log.txt'])

  # specific configuration
  if mode == 'WMS':
    job.base_path = '/vo.cta.in2p3.fr/user/b/bregeon'
    job.ts_task_id = '12'
    simtel_meta_data = {'array_layout': 'full', 'site': 'Paranal',
                        'particle': 'gamma', 'phiP': 0.0, 'thetaP': 20.0}
    job.set_meta_data(simtel_meta_data)
    job.set_file_meta_data(nsb=1, sct="True")
    job.setupWorkflow(debug=True)
    # subtmit to the WMS for debug
    # job.setDestination('LCG.IN2P3-CC.fr')
    result = submit_wms(job)
  elif mode == 'TS':
    input_meta_query = get_dataset_MQ(dataset_name)
    # refine output meta data if needed
    output_meta_data = copy(input_meta_query)
    job.set_meta_data(output_meta_data)
    job.set_file_meta_data(nsb=1, sct="True")
    # adjust calibration file
    job.ts_task_id = '@{JOB_ID}'  # dynamic
    job.setupWorkflow(debug=False)
    job.setType('EvnDisp3')  # mandatory *here*
    result = submit_trans(job, trans_name, input_meta_query, group_size)
  else:
    DIRAC.gLogger.error('1st argument should be the job mode: WMS or TS,\n\
                             not %s' % mode)
    return None

  return result


#########################################################
if __name__ == '__main__':

  arguments = Script.getPositionalArgs()
  if len(arguments) not in [1, 4]:
    Script.showHelp()
  try:
    result = launch_job(arguments)
    if not result['OK']:
      DIRAC.gLogger.error(result['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice('Done')
  except Exception:
    DIRAC.gLogger.exception()
    DIRAC.exit(-1)



"""
  Wrapper on the Job class to handle Simtel Analysis of the PROD4 LST+Magic Corsika
  showers

  https://forge.in2p3.fr/issues/35807
                        April 11th 2019 - J. Bregeon
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
from DIRAC.Interfaces.API.Job import Job
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from CTADIRAC.Core.Utilities.tool_box import DATA_LEVEL_METADATA_ID


class Prod4SimtelLSTMagicJob(Job):
  """ Job extension class for Simtel Analysis

  """

  def __init__(self, cpuTime=432000):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__(self)
    self.setCPUTime(cpuTime)
    # defaults
    self.setName('Simtel')
    self.package = 'corsika_simhessarray'
    self.program_category = 'tel_sim'
    self.version = '2018-11-07-lstmagic'
    self.configuration_id = 5
    self.output_data_level = DATA_LEVEL_METADATA_ID['DL0']
    self.N_output_files = 2  # 1 data and 1 log tar file
    self.base_path = '/vo.cta.in2p3.fr/MC/PROD4/'
    self.fcc = FileCatalogClient()
    self.metadata = collections.OrderedDict()
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])
    self.ts_task_id = 0

  def set_meta_data(self, corsika_md):
    """ Set simtel meta data

    Parameters:
    corsika_md -- metadata dictionary of Corsika files
    """
    # # Set evndisp directory meta data
    self.metadata['array_layout'] = corsika_md['array_layout']
    self.metadata['site'] = corsika_md['site']
    self.metadata['particle'] = corsika_md['particle']
    try:
      phiP = corsika_md['phiP']['=']
    except BaseException:
      phiP = corsika_md['phiP']
    # invert phi from Corsika to Simtel as the reference systems is inverted
    self.metadata['phiP'] = str((float(phiP) + 180) % 360)
    try:
      thetaP = corsika_md['thetaP']['=']
    except BaseException:
      thetaP = corsika_md['thetaP']
    self.metadata['thetaP'] = thetaP
    self.metadata[self.program_category + '_prog'] = 'simtel'
    self.metadata[self.program_category + '_prog_version'] = self.version
    self.metadata['data_level'] = self.output_data_level
    self.metadata['configuration_id'] = self.configuration_id

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """
    # step 1 -- debug
    i_step = 1
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr',
                                  logFile='LS_Init_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_Init' % i_step
      lsStep['Value']['descr_short'] = 'list files in working directory'
      i_step += 1

    # step 2
    swStep = self.setExecutable('cta-prod3-setupsw',
                                arguments='%s %s' % (self.package,
                                                     self.version),
                                logFile='SetupSoftware_Log.txt')
    swStep['Value']['name'] = 'Step%i_SetupSoftware' % i_step
    swStep['Value']['descr_short'] = 'Setup software'
    i_step += 1

    # step 3 - running
    evStep = self.setExecutable('./dirac_prod4_lst-magic_simtel',
                                arguments="--taskid %s" % self.ts_task_id,
                                logFile='Simtel_Log.txt')
    evStep['Value']['name'] = 'Step%i_Simtel' % i_step
    evStep['Value']['descr_short'] = 'Run Simtel'
    i_step += 1

    # step 4 verify merged data
    mgvStep = self.setExecutable('cta-prod-verifysteps',
                                 arguments="generic %0d 1000 './*.tar'" %
                                 (self.N_output_files),
                                 logFile='Verify_Simtel_Log.txt')
    mgvStep['Value']['name'] = 'Step%i_VerifySimtel' % i_step
    mgvStep['Value']['descr_short'] = 'Verify simtel files'
    i_step += 1

    # Step 5 - define meta data, upload file on SE and register in catalogs
    md_json = json.dumps(self.metadata)
    meta_data_field = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                       'particle': 'VARCHAR(128)',
                       'phiP': 'float', 'thetaP': 'float',
                       self.program_category + '_prog': 'VARCHAR(128)',
                       self.program_category + '_prog_version': 'VARCHAR(128)',
                       'data_level': 'int', 'configuration_id': 'int'}
    md_field_json = json.dumps(meta_data_field)

    # Upload and register data
    file_meta_data = {}
    file_md_json = json.dumps(file_meta_data)
    cone10_tag = ''
    if self.metadata['particle'] == 'gamma-diffuse':
      cone10_tag = '_cone10'
    output_pattern = './*lst-magic%s_data.tar' % cone10_tag
    scripts = '../CTADIRAC/Core/scripts/'
    dm_step = self.setExecutable(scripts + 'cta-prod-managedata.py',
                                 arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                 (md_json, md_field_json, file_md_json,
                                  self.base_path, output_pattern, self.package,
                                  self.program_category, self.catalogs),
                                 logFile='DataManagement_Log.txt')
    i_step += 1

    # Upload and register log file (contains histograms)
    file_meta_data = {}
    file_md_json = json.dumps(file_meta_data)
    log_file_pattern = './*_log.tar'
    scripts = '../CTADIRAC/Core/scripts/'
    log_step = self.setExecutable(scripts + 'cta-prod-managedata.py',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                  (md_json, md_field_json, file_md_json,
                                   self.base_path, log_file_pattern, self.package,
                                   self.program_category, self.catalogs),
                                  logFile='LogManagement_Log.txt')
    log_step['Value']['name'] = 'Step%s_LogManagement' % i_step
    log_step['Value']['descr_short'] = 'Save log to SE and register them in DFC'
    i_step += 1

    # step 5 - debug only
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr', logFile='LS_End_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_End' % i_step
      lsStep['Value']['descr_short'] = 'list files in working directory and sub-directory'
      i_step += 1



""" Prod3 MC Pipe Script to create a Transformation
          JB, LA December 2016
"""

__RCSID__ = "$Id$"

from DIRAC.Core.Base import Script
Script.setUsageMessage(
    '\n'.join(
        [
            __doc__.split('\n')[1],
            'Usage:',
            '  %s array_layout site particle pointing_dir zenith_angle nShower' %
            Script.scriptName,
            'Arguments:',
            '  array_layout: hex or square',
            '  site: Paranal, Aar, Armazones_2K',
            '  particle: gamma, proton, electron',
            '  pointing_dir: North or South',
            '  zenith_agle: 20',
            '  nShower: from 5 to 25000',
            '\ne.g: %s hex Paranal gamma South 20 5' %
            Script.scriptName,
        ]))

Script.parseCommandLine()

import DIRAC
from DIRAC import gLogger
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.Core.Workflow.Parameter import Parameter
from CTADIRAC.Interfaces.API.Prod3MCPipeJob import Prod3MCPipeJob


def submitTS(job):
  """ Create a transformation executing the job workflow  """

  ### Temporary fix to initialize JOB_ID #######
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "", True, False, "Temporary fix"))
  job.workflow.addParameter(Parameter("PRODUCTION_ID", "000000", "string", "", "", True, False, "Temporary fix"))
  job.setType('MCSimulation')  # Used for the JobType plugin

  t = Transformation()
  # t.setTransformationName( "Prod3Exemple" )  # This must be unique. If not set it's asked in the prompt
  t.setType("MCSimulation")
  t.setDescription("MC prod3 example")
  t.setLongDescription("corsika-simtel production")  # mandatory
  t.setBody(job.workflow.toXML())

  res = t.addTransformation()  # Transformation is created here

  if not res['OK']:
    gLogger.error(res['Message'])
    DIRAC.exit(-1)

  t.setStatus("Active")
  t.setAgentType("Automatic")

  return res

#########################################################


def runProd3(args=None):
  """ Simple wrapper to create a Prod3MCJob and setup parameters
      from positional arguments given on the command line.

      Parameters:
      args -- a list of 6 strings corresponding to job arguments
              array_layout site particle pointing_dir zenith_angle nShower
  """
  # get arguments
  layout = args[0]
  site = args[1]
  particle = args[2]
  pointing = args[3]
  zenith = args[4]
  nShower = args[5]

  ### Main Script ###
  job = Prod3MCPipeJob()

  # override for testing
  job.setName('Prod3b_LaPalma3_60deg_%s' % particle)

  # package and version
  job.setPackage('corsika_simhessarray')
  job.setVersion('2016-12-20c')  # final with fix for gamma-diffuse
  job.no_sct = True  # NO SCT for 40deg and 60deg !

  # layout, site, particle, pointing direction, zenith angle
  # lapalma3, LaPalma,  gamma, South,  20
  job.setArrayLayout(layout)
  job.setSite(site)
  job.setParticle(particle)
  job.setPointingDir(pointing)
  job.setZenithAngle(zenith)

  # 5 is enough for testing
  job.setNShower(nShower)

  # Set the startrunNb here (it will be added to the Task_ID)
  startrunNb = '0'
  job.setStartRunNumber(startrunNb)

  # set run number for TS submission: JOB_ID variable left for dynamic
  # resolution during the Job. It corresponds to the Task_ID
  job.setRunNumber('@{JOB_ID}')

  # get dirac log files
  job.setOutputSandbox(['*Log.txt'])

  # add the sequence of executables
  job.setupWorkflow(debug=False)

  # submit to the Transformation System
  res = submitTS(job)

  # debug
  Script.gLogger.info(job.workflow)

  return res


#########################################################
if __name__ == '__main__':

  args = Script.getPositionalArgs()
  if (len(args) not in [6, 7]):
    Script.showHelp()
  try:
    res = runProd3(args)
    if not res['OK']:
      DIRAC.gLogger.error(res['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice('Done')
  except Exception:
    DIRAC.gLogger.exception()
    DIRAC.exit(-1)



import json


class DMSEncoder(json.JSONEncoder):
  """ This class is an encoder for the Requests, Operation and Files.
  """

  def default(self, obj):  # pylint: disable=method-hidden

    if hasattr(obj, '_getJSONData'):
      return obj._getJSONData()
    else:
      return json.JSONEncoder.default(self, obj)



#!/usr/bin/env python

"""
  Get transformations with a given Status
"""

__RCSID__ = "$Id$"

import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

@Script()
def main():

  Script.registerSwitch('', 'Status=', 'status')
  Script.parseCommandLine()

  status = None

  for switch in Script.getUnprocessedSwitches():
    if switch[0] == 'Status':
      status = switch[1].capitalize()

  tc = TransformationClient()
  res = tc.getTransformationWithStatus(status)

  if not res['OK']:
    DIRAC.gLogger.error(res['Message'])
    DIRAC.exit(-1)

  transIDs = res['Value']

  if transIDs:
    DIRAC.gLogger.notice("Found %d transformations with status %s" % (len(transIDs), status))
  else:
    DIRAC.gLogger.notice("No transformations selected with status", status)
    DIRAC.exit(0)

  strTransIDs = []
  for transID in transIDs:
    strTransIDs.append(str(transID))

  DIRAC.gLogger.notice("%s" % ','.join(strTransIDs))

  DIRAC.exit(0)

if __name__ == "__main__":
  main()


""" Prod3 MC Pipe Script to run realistic test jobs
          JB, March 2020
"""

from DIRAC.Core.Base import Script
Script.setUsageMessage(
    '\n'.join(
        [
            __doc__.split('\n')[1],
            'Usage:',
            '  %s array_layout site particle pointing_dir zenith_angle nShower' %
            Script.scriptName,
            'Arguments:',
            '  mode: WMS or TS',
            '  array_layout: demo',
            '  site: Paranal or LaPalma',
            '  particle: gamma, proton, electron',
            '  pointing_dir: North or South',
            '  zenith_agle: 20',
            '  nShower: from 5 to 25000',
            '\ne.g: %s TS Baseline Paranal gamma South 20 5' %
            Script.scriptName,
        ]))

Script.parseCommandLine()

import DIRAC
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.Core.Workflow.Parameter import Parameter
from CTADIRAC.Interfaces.API.Prod3MCPipeTestJob import Prod3MCPipeTestJob
from DIRAC.Interfaces.API.Dirac import Dirac


def submit_trans(job, trans_name):
  """ Create a transformation executing the job workflow  """
  DIRAC.gLogger.notice('submit_trans : %s' % trans_name)

  # Initialize JOB_ID
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))
  job.workflow.addParameter(Parameter("PRODUCTION_ID", "000000", "string",
                                      "", "", True, False, "Temporary fix"))
  job.setType('MCSimulation')  # Used for the JobType plugin

  trans = Transformation()
  trans.setTransformationName(trans_name)
  trans.setType("MCSimulation")
  trans.setDescription("MC Prod3 BaseLine Corsika7 test")
  trans.setLongDescription("corsika-simtel production")  # mandatory
  trans.setBody(job.workflow.toXML())
  result = trans.addTransformation()  # transformation is created here
  if not result['OK']:
    return result
  trans.setStatus("Active")
  trans.setAgentType("Automatic")
  trans_id = trans.getTransformationID()
  return trans_id


def submit_wms(job):
  """ Submit the job to the WMS
  @todo launch job locally
  """
  dirac = Dirac()
  job.setJobGroup('Prod3MCPipeTestJob')
  result = dirac.submitJob(job)
  if result['OK']:
    Script.gLogger.notice('Submitted job: ', result['Value'])
  return result

#########################################################


def run_prod3(args=None):
  """ Simple wrapper to create a Prod3MCPipeBaselineJob and setup parameters
    from positional arguments given on the command line.

    Parameters:
    args -- a list of 6 strings corresponding to job arguments
            array_layout site particle pointing_dir zenith_angle nShower
            demo LaPalma  gamma South 20 1000
  """
  # get arguments
  mode = args[0]
  layout = args[1]
  site = args[2]
  particle = args[3]
  pointing = args[4]
  zenith = args[5]
  nShower = args[6]

  ### Main Script ###
  job = Prod3MCPipeTestJob()

  # override for testing
  job.setName('BL_Corsika7_Test_Paranal_20deg_%s' % particle)

  # package and version
  job.setPackage('corsika_simtelarray')
  job.setVersion('2019-09-03')  # final with fix for gamma-diffuse
  job.configuration_id = -1

  # layout, site, particle, pointing direction, zenith angle
  # demo, LaPalma,  gamma, South,  20
  job.setArrayLayout(layout)
  job.setSite(site)
  job.setParticle(particle)
  job.setPointingDir(pointing)
  job.setZenithAngle(zenith)

  # 5 is enough for testing
  job.setNShower(nShower)

  # Set the startrunNb here (it will be added to the Task_ID)
  startrunNb = '0'
  job.setStartRunNumber(startrunNb)

  # set run number for TS submission: JOB_ID variable left for dynamic
  # resolution during the Job. It corresponds to the Task_ID
  job.setRunNumber('@{JOB_ID}')

  # get dirac log files
  job.setOutputSandbox(['*Log.txt'])

  # specific configuration
  if mode == 'WMS':
    job.base_path = '/vo.cta.in2p3.fr/user/b/bregeon'
    job.start_run_number = '0'
    job.run_number = '1001'
    job.setupWorkflow(debug=True)
    # subtmit to the WMS for debug
    job.setDestination('LCG.IN2P3-CC.fr')
    # job.setDestination('LCG.CIEMAT.es')
    result = submit_wms(job)
  elif mode == 'TS':
    job.base_path = '/vo.cta.in2p3.fr/MC/PRODTest/'
    job.start_run_number = '0'
    job.run_number = '@{JOB_ID}'  # dynamic
    job.setupWorkflow(debug=False)
    job.setType('MCSimulation')
    tag = ''  # _test9_MCSimPostWorkflow_5k'
    trans_name = 'MC_Prod3_Test_%s_%s_%s_%s%s' %\
        (job.cta_site, job.particle, job.pointing_dir, job.zenith_angle, tag)
    result = submit_trans(job, trans_name)
  else:
    DIRAC.gLogger.error('1st argument should be the job mode: WMS or TS,\n\
                             not %s' % mode)
    return None

  return result


#########################################################
if __name__ == '__main__':

  arguments = Script.getPositionalArgs()
  if len(arguments) != 7:
    Script.showHelp()
  try:
    result = run_prod3(arguments)
    if not result['OK']:
      DIRAC.gLogger.error(result['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice('Done')
  except Exception:
    DIRAC.gLogger.exception()
    DIRAC.exit(-1)



"""
  Simple Wrapper on the Job class to handle Prod3 MC
  piping corsika into simtels
  created for Prod3 tests with reference base line
  for both La Palma and Paranal
			JB,LA April 2017
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job

class Prod3MCPipeBaselineJob( Job ) :
  """ Job extension class for Prod3 MC simulations,
      takes care of running corsika piped into simtel
      one output file is created
  """

  def __init__( self, cpuTime = 259200 ):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__( self )
    self.setCPUTime( cpuTime )
    # defaults
    self.setName('Prod3MC_Generation')
    self.package='corsika_simhessarray'
    self.program_category='tel_sim'
    self.version='2017-04-19'
    self.configuration_id = 0
    self.output_data_level=0
    self.nShower=100
    self.start_run_number = '0'
    self.run_number = '10'
    self.cta_site='Paranal'
    self.cta_site_tag=''
    self.array_layout = 'Baseline'
    self.particle='gamma'
    self.pointing_dir = 'South'
    self.zenith_angle = 20.
    #self.inputpath = 'Data/sim_telarray/cta-prod3-%s/0.0deg'%self.array_layout.lower()
    self.inputpath = 'Data/sim_telarray/cta-prod3-demo/0.0deg'
    self.N_output_files = 1
    self.basepath = '/vo.cta.in2p3.fr/MC/PROD3/'
    self.catalogs = json.dumps(['DIRACFileCatalog','TSCatalog'])

  def setPackage(self, package):
    """ Set package name : e.g. 'corsika_simhessarray'

    Parameters:
    package -- corsika_simhessarray
    """
    self.package=package

  def setVersion(self, version):
    """ Set software version number : e.g. 2016-12-02

    Parameters:
    version -- corsika+simtel package version number
    """
    self.version=version

  def setNShower(self, nshow):
    """ Set the number of corsika showers,
        5 is enough for testing
        20000 in production usually.

    Parameters:
    nshow -- number of corsika primary showers to generate
    """
    self.nShower=nshow

  def setRunNumber(self, runNb):
    """ Set the corsika run number, passed as a string
        because may be a TS parameter

    Parameters:
    runNb -- run number as a string, used as a corsika seed
    """
    self.run_number=runNb

  def setStartRunNumber( self, startrunNb ):
    """ Set the corsika start run number (to be added to the run_number), passed as a string
        because may be a TS parameter

    Parameters:
    startrunNb -- to be added to the run number
    """
    self.start_run_number = startrunNb

  def setArrayLayout(self, layout):
    """ Set the array layout type

    Parameters:
    layout -- a string for the array layout, hex or square
    """
    if layout in ['Baseline']:
        DIRAC.gLogger.info ( 'Set Simtel layout to: %s'%layout )
        self.array_layout=layout
    else:
        DIRAC.gLogger.error ( 'Unknown layout: : %s'% layout )
        DIRAC.exit(-1)

  def setSite(self, site):
    """ Set the site to simulate

    Parameters:
    site -- a string for the site name (LaPalma)
    """
    self.cta_site=site

  def setParticle(self, particle):
    """ Set the corsika primary particle

    Parameters:
    particle -- a string for the particle type/name
    """
    if particle in ['gamma','gamma-diffuse','electron','proton','helium']:
        DIRAC.gLogger.info ( 'Set Corsika particle to: %s'%particle )
        self.particle=particle
    else:
        DIRAC.gLogger.error ( 'Corsika does not know particle type: %s'%particle )
        DIRAC.exit(-1)

  def setPointingDir(self, pointing):
    """ Set the pointing direction, North or South

    Parameters:
    pointing -- a string for the pointing direction
    """
    if pointing in ['North', 'South', 'East', 'West']:
        DIRAC.gLogger.info ( 'Set Pointing dir to: %s'%pointing )
        self.pointing_dir=pointing
    else:
        DIRAC.gLogger.error ( 'Unknown pointing direction: %s'%pointing )
        DIRAC.exit(-1)

  def setZenithAngle( self, zenith ):
    """ Set the pointing direction, North or South

    Parameters:
    zenith -- a string for the zenith angle
    """
    self.zenith_angle = zenith

  def setnOutputFiles(self, N):
    """ Set the number of expected output data files

    Parameters:
    N -- an int for the number of data files
    """
    self.N_output_files = N


  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """
    # step 1 - debug only
    iStep = 1
    if debug:
        lsStep = self.setExecutable( '/bin/ls -alhtr', logFile = 'LS_Init_Log.txt' )
        lsStep['Value']['name']='Step%i_LS_Init'%iStep
        lsStep['Value']['descr_short']='list files in working directory'
        iStep+=1

    # step 2
    swStep = self.setExecutable( 'cta-prod3-setupsw',
                              arguments='%s %s'% (self.package, self.version),\
                              logFile='SetupSoftware_Log.txt')
    swStep['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    swStep['Value']['descr_short'] = 'Setup software'
    iStep+=1

    # step 3
    if self.cta_site == 'Paranal':
        prod_script='./dirac_prod3_paranal_baseline'
    elif self.cta_site == 'LaPalma':
        prod_script='./dirac_prod3_lapalma_baseline'
    else:
        DIRAC.gLogger.error('Unknown site: %s'%self.cta_site)
        DIRAC.gLogger.error('No shell script associated')
        DIRAC.exit(-1)

    csStep = self.setExecutable( prod_script, \
                              arguments = '--start_run %s --run %s %s %s %s %s' % \
                                         ( self.start_run_number, self.run_number, \
                                           self.cta_site,\
                                           self.particle, self.pointing_dir, self.zenith_angle ), \
                              logFile='CorsikaSimtel_Log.txt')
    csStep['Value']['name']='Step%i_CorsikaSimtel'%iStep
    csStep['Value']['descr_short']='Run Corsika piped into simtel'
    iStep+=1

    # step 4 verify merged data
    mgvStep = self.setExecutable( 'cta-prod-verifysteps', \
                              arguments = "generic %0d 1000 '%s/Data/*.simtel.gz'"%\
                                          (self.N_output_files, self.inputpath),\
                              logFile='Verify_Simtel_Log.txt')
    mgvStep['Value']['name']='Step%i_VerifySimtel'%iStep
    mgvStep['Value']['descr_short'] = 'Verify simtel files'
    iStep += 1

    # step 5 - debug only
    if debug:
        lsStep=self.setExecutable('/bin/ls -alhtr Data/sim_telarray/*/*/*',logFile='LS_End_Log.txt')
        lsStep['Value']['name']='Step%i_LS_End'%iStep
        lsStep['Value']['descr_short']='list files in working directory and sub-directory'
        iStep += 1

    # step 6
    # ## the order of the metadata dictionary is important, since it's used to build the directory structure
    metadata = collections.OrderedDict()
    metadata['array_layout'] = self.array_layout
    metadata['site'] = self.cta_site
    metadata['particle'] = self.particle
    if self.pointing_dir == 'North':
      metadata['phiP'] = 180
    if self.pointing_dir == 'South':
      metadata['phiP'] = 0
    metadata['thetaP'] = float( self.zenith_angle )
    metadata['tel_sim_prog'] = 'simtel'
    metadata['tel_sim_prog_version'] = self.version
    metadata['data_level'] = self.output_data_level
    metadata['configuration_id'] = self.configuration_id
    mdjson = json.dumps( metadata )

    metadatafield = {'array_layout':'VARCHAR(128)', 'site':'VARCHAR(128)',
                     'particle':'VARCHAR(128)', 'phiP':'float',
                     'thetaP': 'float',
                     'tel_sim_prog':'VARCHAR(128)',
                     'tel_sim_prog_version':'VARCHAR(128)',
                     'data_level': 'int', 'configuration_id': 'int'}

    mdfieldjson = json.dumps( metadatafield )

    filemetadata = {'runNumber': self.run_number}

    fmdjson = json.dumps( filemetadata )

    dmStep = self.setExecutable('cta-prod3-managedata',
                                arguments="'%s' '%s' '%s' %s %s %s '%s'" %
                                (mdjson, mdfieldjson, fmdjson, self.inputpath,
                                 self.basepath, self.start_run_number, self.catalogs),
                                logFile='DataManagement_Log.txt')
    dmStep['Value']['name'] = 'Step%i_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save files to SE and register them in DFC'
    iStep += 1

    # Number of showers is passed via an environment variable
    self.setExecutionEnv( {'NSHOW'        : '%s' % self.nShower} )



"""
  Simple Wrapper on the Job class to handle EvnDisp Analysis
  for the Prod3 SCT 2020 analysis from container
  https://forge.in2p3.fr/issues/41709

  November 16th 2020 - J. Bregeon
                  bregeon@in2p3.fr
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
from DIRAC.Interfaces.API.Job import Job


class EvnDispProd3SCTSingJob(Job):
  """ Job extension class for EvnDisp Analysis,
    takes care of running converter and evndisp.
  """

  def __init__(self, cpuTime=432000):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__(self)
    self.setCPUTime(cpuTime)
    # defaults
    self.setName('Evndisplay_CalibReco')
    self.package = 'evndisplay'
    self.version = 'eventdisplay-cta-dl1-prod3b-sct.v02'
    self.container = True
    self.program_category = 'calibimgreco'
    self.prog_name = 'evndisp'
    self.configuration_id = 9
    self.output_data_level = 1
    self.base_path = '/vo.cta.in2p3.fr/MC/PROD3/'
    self.metadata = collections.OrderedDict()
    self.file_meta_data = dict()
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])
    self.ts_task_id = 0

  def set_meta_data(self, tel_sim_md):
    """ Set EventDisplay meta data

    Parameters:
    tel_sim_md -- metadata dictionary from the telescope simulation
    """
    # # Set evndisp directory meta data
    self.metadata['array_layout'] = tel_sim_md['array_layout']
    self.metadata['site'] = tel_sim_md['site']
    self.metadata['particle'] = tel_sim_md['particle']
    try:
      phiP = tel_sim_md['phiP']['=']
    except TypeError:
      phiP = tel_sim_md['phiP']
    self.metadata['phiP'] = phiP
    try:
      thetaP = tel_sim_md['thetaP']['=']
    except TypeError:
      thetaP = tel_sim_md['thetaP']
    self.metadata['thetaP'] = thetaP
    self.metadata[self.program_category + '_prog'] = self.prog_name
    self.metadata[self.program_category + '_prog_version'] = self.version
    self.metadata['data_level'] = self.output_data_level
    self.metadata['configuration_id'] = self.configuration_id

  def set_file_meta_data(self, nsb=1, sct="True"):
    """ Set evndisplay file meta data

    Parameters:
    meta_data_dict -- metadata dictionary
    """
    # Set evndisp file meta data
    self.file_meta_data['nsb'] = nsb
    self.file_meta_data['sct'] = sct

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """
    i_step = 0
    # step 1 -- debug
    if debug:
      ls_step = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
      ls_step['Value']['name'] = 'Step%i_LS_Init' % i_step
      ls_step['Value']['descr_short'] = 'list files in working directory'
      i_step += 1

      env_step = self.setExecutable('/bin/env', logFile='Env_Log.txt')
      env_step['Value']['name'] = 'Step%i_Env' % i_step
      env_step['Value']['descr_short'] = 'Dump environment'
      i_step += 1

    # # step 2
    # sw_step = self.setExecutable('cta-prod-setup-software',
    #                           arguments='-p %s -v %s -a simulations -g %s'%
    #                           (self.package, self.version, self.compiler),\
    #                           logFile='SetupSoftware_Log.txt')
    # sw_step['Value']['name'] = 'Step%i_SetupSoftware' % i_step
    # sw_step['Value']['descr_short'] = 'Setup software'
    # i_step+=1

    # step 3 verify input data size
    # arguments are nbFiles=0 (not used) and fileSize=1000kB
    eiv_step = self.setExecutable('cta-prod-verifysteps',
                                  arguments="generic 1 1000 '*.simtel.gz'",
                                  logFile='Verify_EvnDispInputs_Log.txt')
    eiv_step['Value']['name'] = 'Step%i_VerifyEvnDispInputs' % i_step
    eiv_step['Value']['descr_short'] = 'Verify EvnDisp Inputs'
    i_step += 1

    # step 4 run EventDisplay
    ev_step = self.setExecutable('singularity',
                                 arguments='run /cvmfs/cta.in2p3.fr/software/containers/%s \
                                    /evndisp/dirac_sing_evndisp.sh' %
                                 (self.version),
                                 logFile='EvnDisp_Log.txt')
    ev_step['Value']['name'] = 'Step%i_EvnDisplay' % i_step
    ev_step['Value']['descr_short'] = 'Run EvnDisplay'
    i_step += 1

    # step 5 set meta data and register Data
    meta_data_json = json.dumps(self.metadata)
    file_meta_data_json = json.dumps(self.file_meta_data)

    meta_data_field = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                       'particle': 'VARCHAR(128)',
                       'phiP': 'float', 'thetaP': 'float',
                       self.program_category + '_prog': 'VARCHAR(128)',
                               self.program_category + '_prog_version': 'VARCHAR(128)',
                       'data_level': 'int', 'configuration_id': 'int'}
    meta_data_field_json = json.dumps(meta_data_field)

    # register Data
    data_output_pattern = './Data/*.simtel.gz.DL1.root'
    scripts = '../CTADIRAC/Core/scripts/'
    dm_step = self.setExecutable(scripts + 'cta-prod-managedata.py',
                                 arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                 (meta_data_json, meta_data_field_json,
                                  file_meta_data_json,
                                  self.base_path, data_output_pattern, self.package,
                                  self.program_category, self.catalogs),
                                 logFile='DataManagement_Log.txt')
    dm_step['Value']['name'] = 'Step%s_DataManagement' % i_step
    dm_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    i_step += 1

    # register Data lin
    data_output_pattern = './Data/*.simtel.gz.lin.DL1.root'
    dm2_step = self.setExecutable(scripts + 'cta-prod-managedata.py',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                  (meta_data_json, meta_data_field_json,
                                   file_meta_data_json,
                                   self.base_path, data_output_pattern, self.package,
                                   self.program_category, self.catalogs),
                                  logFile='DataManagement2_Log.txt')
    dm2_step['Value']['name'] = 'Step%s_DataManagement' % i_step
    dm2_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    i_step += 1

    # step 7 failover step
    if not debug:
      failover_step = self.setExecutable('/bin/ls -l',
                                         modulesList=['Script', 'FailoverRequest'])
      failover_step['Value']['name'] = 'Step%s_Failover' % i_step
      failover_step['Value']['descr_short'] = 'Tag files as unused if job failed'
      i_step += 1

    # Step 8 - debug only
    if debug:
      ls_step = self.setExecutable('/bin/ls -Ralhtr', logFile='LS_End_Log.txt')
      ls_step['Value']['name'] = 'Step%s_LSHOME_End' % i_step
      ls_step['Value']['descr_short'] = 'list files in Home directory'
      i_step += 1



#!/usr/bin/env python
"""
  Create a Transformation for bulk data replication based on Catalog Query
"""

__RCSID__ = "$Id"

import DIRAC
from DIRAC.Core.Base import Script

Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s [option|cfgfile] ...' % Script.scriptName]))

Script.parseCommandLine()

from DIRAC.TransformationSystem.Client.Transformation import Transformation


def DataReplicationByQueryTSExample(args=None):

  t = Transformation()

  t.setType("Replication")
  t.setSourceSE(['CYF-STORM-Disk', 'DESY-ZN-Disk'])  # A list of SE where at least 1 SE is the valid one
  t.setTargetSE(['CC-IN2P3-Disk'])
  t.setDescription("data Replication")
  t.setLongDescription("data Replication")  # mandatory

  t.setGroupSize(10)

  t.setPlugin("Broadcast")

  inputQuery = {
      'MCCampaign': 'PROD4',
      'site': 'Paranal',
      'calibimgreco_prog': 'evndisp',
      'calibimgreco_prog_version': 'prod4_d20181110',
      'particle': 'gamma',
      'array_layout': 'Baseline-SST-only',
      'configuration_id': 4,
      'phiP': 180.0,
      'thetaP': 20.0,
      'outputType': 'Data',
      'data_level': 1}
  t.setInputMetaQuery(inputQuery)

  res = t.addTransformation()  # transformation is created here
  if not res['OK']:
    DIRAC.gLogger.error(res['Message'])
    DIRAC.exit(-1)

  # t.setStatus("Active")
  # t.setAgentType("Automatic")


if __name__ == '__main__':

  args = Script.getPositionalArgs()

  try:
    DataReplicationByQueryTSExample(args)
  except Exception:
    Script.gLogger.exception()



#!/usr/bin/env python
""" Simple data management script for Analysis PROD3 MC
    create DFC MetaData structure
    put and register files in DFC
"""

__RCSID__ = "$Id$"

# generic imports
import os
import glob
import json

# DIRAC imports
import DIRAC
from DIRAC.Core.Base import Script
Script.parseCommandLine()

# Specific DIRAC imports
from CTADIRAC.Core.Workflow.Modules.ProdDataManager import ProdDataManager


def getRunNumber(filename, package):
    if filename[-9:] == '.logs.tgz':
        run_number = int(filename.split('/')[-1].split('_')[-1].split('.')[0])
    elif package in ['chimp', 'mars', 'corsika_simhessarray', 'corsika_simtelarray']:
        run_number = int(filename.split('run')[1].split('___cta')[0])
    elif package == 'evndisplay':
        if filename[-8:] in ['DL1.root', 'DL2.root']:
            run_number = int(filename.split('run')[1].split('___cta')[0])
        elif filename[-10:] in ['DL1.tar.gz', 'DL2.tar.gz']:
            run_number = int(filename.split('run')[1].split('___cta')[0])
        else:
            run_number = int(filename.split('-')[0])  # old default
    elif package == 'ctapipe':
        run_number = filename.split('run')[1].split('.h5')[0]
        if '-' in run_number:
          run_number = run_number.split('-')[0]
    return str(run_number)

####################################################
def main():
    """ simple wrapper to put and register all analysis files

    Keyword arguments:
    args -- a list of arguments in order []
    """
    args = Script.getPositionalArgs()
    metadata = args[0]
    metadatafield = args[1]
    filemetadata = args[2]
    basepath = args[3]
    outputpattern = args[4]
    package = args[5]
    program_category = args[6]
    catalogs = args[7]
    catalogsjson = json.loads( catalogs )
    if len(args)==8:
      outputType='Data'
    else:
      outputType='Log'

    # # Create MD structure
    prod3dm = ProdDataManager( catalogsjson )
    res = prod3dm.createMDStructure( metadata, metadatafield, basepath, program_category)
    if res['OK']:
      path = res['Value']
    else:
      return res

    # # Upload data files
    res = prod3dm._checkemptydir( outputpattern )
    if not res['OK']:
      return res

    for localfile in glob.glob( outputpattern ):
      filename = os.path.basename( localfile )
      run_number = getRunNumber( filename, package )
      runpath = prod3dm._getRunPath( run_number )
      lfn = os.path.join( path, outputType, runpath, filename )
      res = prod3dm.putAndRegister( lfn, localfile, filemetadata, package )
      if not res['OK']:
        return res

    DIRAC.exit()

####################################################
if __name__ == '__main__':
  main()


#!/bin/env python
"""
Show request given its ID, a jobID or a transformation and a task
Extension of dirac-rms-request with additional options:
--TaskStatus: selection of requests associated to a Transformation based on TaskStatus
--Stat: to print only summary informations
"""
__RCSID__ = "$Id$"

import datetime
import os

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

Script.registerSwitch('', 'Job=', '   JobID[,jobID2,...]')
Script.registerSwitch('', 'Transformation=', '   transformation ID')
Script.registerSwitch('', 'Tasks=', '      Associated to --Transformation, list of taskIDs')
Script.registerSwitch('', 'TaskStatus=', '      Associated to --Transformation')
Script.registerSwitch('', 'Stat', '   Only print summary informations')
Script.registerSwitch('', 'Verbose', '   Print more information')
Script.registerSwitch('', 'Terse', '   Only print request status')
Script.registerSwitch('', 'Full', '   Print full request content')
Script.registerSwitch('', 'Status=', '   Select all requests in a given status')
Script.registerSwitch('', 'Since=',
                      '      Associated to --Status, start date yyyy-mm-dd or nb of days (default= -one day')
Script.registerSwitch('', 'Until=', '      Associated to --Status, end date (default= now')
Script.registerSwitch('', 'Maximum=', '      Associated to --Status, max number of requests ')
Script.registerSwitch('', 'Reset', '   Reset Failed files to Waiting if any')
Script.registerSwitch('', 'Force', '   Force reset even if not Failed')
Script.registerSwitch('', 'All', '      (if --Status Failed) all requests, otherwise exclude irrecoverable failures')
Script.registerSwitch('', 'FixJob', '   Set job Done if the request is Done')
Script.registerSwitch('', 'Cancel', '   Cancel the request')
Script.registerSwitch('', 'ListJobs', ' List the corresponding jobs')
Script.registerSwitch('', 'TargetSE=', ' Select request only if that SE is in the targetSEs')
Script.setUsageMessage('\n'.join([__doc__,
                                  'Usage:',
                                  ' %s [option|cfgfile] [request[,request1,...]|<file>' % Script.scriptName,
                                  'Arguments:',
                                  ' request: a request ID or a unique request name',
                                  ' <file>: a file containing a list of requests (Comma-separated on each line)']))

Script.parseCommandLine()

def convertDate(date):
  try:
    value = datetime.datetime.strptime(date, '%Y-%m-%d')
    return value
  except Exception:
    pass
  try:
    value = datetime.datetime.utcnow() - datetime.timedelta(hours=int(24 * float(date)))
  except Exception:
    gLogger.fatal("Invalid date", date)
    value = None
  return value

@Script()
def main():

  jobs = []
  requestID = 0
  transID = None
  taskIDs = None
  tasks = None
  taskStatus = None
  requests = []
  full = False
  verbose = False
  stat = False
  status = None
  until = None
  since = None
  terse = False
  allR = False
  reset = False
  fixJob = False
  maxRequests = 999999999999
  cancel = False
  listJobs = False
  force = False
  targetSE = set()
  for switch in Script.getUnprocessedSwitches():
    if switch[0] == 'Job':
      try:
        jobs = []
        for arg in switch[1].split(','):
          if os.path.exists(arg):
            lines = open(arg, 'rt').readlines()
            jobs += [int(job.strip()) for line in lines for job in line.split(',')]
            gLogger.notice("Found %d jobs in file %s" % (len(jobs), arg))
          else:
            jobs.append(int(arg))
      except TypeError:
        gLogger.fatal("Invalid jobID")
    elif switch[0] == 'Transformation':
      try:
        transID = int(switch[1])
      except Exception:
        gLogger.fatal('Invalid transID', switch[1])
    elif switch[0] == 'Tasks':
      try:
        taskIDs = [int(task) for task in switch[1].split(',')]
      except Exception:
        gLogger.fatal('Invalid tasks', switch[1])
    elif switch[0] == 'TaskStatus':
      taskStatus = switch[1].capitalize()
    elif switch[0] == 'Stat':
      stat = True
    elif switch[0] == 'Full':
      full = True
    elif switch[0] == 'Verbose':
      verbose = True
    elif switch[0] == 'Terse':
      terse = True
    elif switch[0] == 'All':
      allR = True
    elif switch[0] == 'Reset':
      reset = True
    elif switch[0] == 'Force':
      force = True
    elif switch[0] == 'Status':
      status = switch[1].capitalize()
    elif switch[0] == 'Since':
      since = convertDate(switch[1])
    elif switch[0] == 'Until':
      until = convertDate(switch[1])
    elif switch[0] == 'FixJob':
      fixJob = True
    elif switch[0] == 'Cancel':
      cancel = True
    elif switch[0] == 'ListJobs':
      listJobs = True
    elif switch[0] == 'Maximum':
      try:
        maxRequests = int(switch[1])
      except Exception:
        pass
    elif switch[0] == 'TargetSE':
      targetSE = set(switch[1].split(','))

  if reset and not force:
    status = 'Failed'
  if fixJob:
    status = 'Done'
  if terse:
    verbose = True
  if status:
    if not until:
      until = datetime.datetime.utcnow()
    if not since:
      since = until - datetime.timedelta(hours=24)
  from DIRAC.RequestManagementSystem.Client.ReqClient import ReqClient
  from DIRAC.RequestManagementSystem.Client.ReqClient import printRequest, recoverableRequest
  from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
  reqClient = ReqClient()
  transClient = TransformationClient()

  if transID:
    if not taskIDs:
      condDict = {'TransformationID': transID}
      if taskStatus:
        condDict.update({'ExternalStatus': taskStatus})
      res = transClient.getTransformationTasks(condDict)
      for record in res['Value']:
        task = record['TaskID']
        requestID = record['ExternalID']
        requests.append(requestID)
      if not requests:
        gLogger.error("No request found for transformation %s" % transID)
        DIRAC.exit(0)
    else:
      requests = ['%08d_%08d' % (transID, task) for task in taskIDs]
    allR = True

  elif not jobs:
    requests = []
    # Get full list of arguments, with and without comma
    for arg in [x.strip() for arg in Script.getPositionalArgs() for x in arg.split(',')]:
      if os.path.exists(arg):
        lines = open(arg, 'rt').readlines()
        requests += [reqID.strip() for line in lines for reqID in line.split(',')]
        gLogger.notice("Found %d requests in file" % len(requests))
      else:
        requests.append(arg)
      allR = True
  else:
    res = reqClient.getRequestIDsForJobs(jobs)
    if not res['OK']:
      gLogger.fatal("Error getting request for jobs", res['Message'])
      DIRAC.exit(2)
    if res['Value']['Failed']:
      gLogger.error("No request found for jobs %s" % ','.join(sorted(str(job) for job in res['Value']['Failed'])))
    requests = sorted(res['Value']['Successful'].values())
    if requests:
      allR = True
    else:
      DIRAC.exit(0)

  if status and not requests:
    allR = allR or status != 'Failed'
    res = reqClient.getRequestIDsList([status], limit=maxRequests, since=since, until=until)

    if not res['OK']:
      gLogger.error("Error getting requests:", res['Message'])
      DIRAC.exit(2)
    requests = [reqID for reqID, _st, updTime in res['Value'] if updTime > since and updTime <= until and reqID]
    gLogger.notice('Obtained %d requests %s between %s and %s' % (len(requests), status, since, until))
  if not requests:
    gLogger.notice('No request selected....')
    Script.showHelp(exitCode=2)
  okRequests = []
  warningPrinted = False
  jobIDList = []
  for reqID in requests:
    # We allow reqID to be the requestName if it is unique
    try:
      requestID = int(reqID)
    except ValueError:
      requestID = reqClient.getRequestIDForName(reqID)
      if not requestID['OK']:
        gLogger.notice(requestID['Message'])
        continue
      requestID = requestID['Value']

    request = reqClient.peekRequest(requestID)
    if not request["OK"]:
      gLogger.error(request["Message"])
      DIRAC.exit(-1)

    request = request["Value"]
    if not request:
      gLogger.error("no such request %s" % requestID)
      continue
    # If no operation as the targetSE, skip
    if targetSE:
      found = False
      for op in request:
        if op.TargetSE and targetSE.intersection(op.TargetSE.split(',')):
          found = True
          break
      if not found:
        continue
    # keep a list of jobIDs if requested
    if request.JobID and listJobs:
      jobIDList.append(request.JobID)

    if status and request.Status != status:
      gLogger.notice("Request %s is not in requested status %s%s" %
                     (reqID, status, ' (cannot be reset)' if reset else ''))
      continue

    if fixJob and request.Status == 'Done' and request.JobID:
      # The request is for a job and is Done, verify that the job is in the proper status
      result = reqClient.finalizeRequest(request.RequestID, request.JobID, useCertificates=False)
      if not result['OK']:
        gLogger.error("Error finalizing job", result['Message'])
      else:
        gLogger.notice("Job %d updated to %s" % (request.JobID, result['Value']))
      continue

    if cancel:
      if request.Status not in ('Done', 'Failed'):
        ret = reqClient.cancelRequest(requestID)
        if not ret['OK']:
          gLogger.error("Error canceling request %s" % reqID, ret['Message'])
        else:
          gLogger.notice("Request %s cancelled" % reqID)
      else:
        gLogger.notice("Request %s is in status %s, not cancelled" % (reqID, request.Status))

    elif allR or recoverableRequest(request):
      okRequests.append(str(requestID))
      if reset:
        gLogger.notice('============ Request %s =============' % requestID)
        ret = reqClient.resetFailedRequest(requestID, allR=allR)
        if not ret['OK']:
          gLogger.error("Error resetting request %s" % requestID, ret['Message'])
      else:
        if len(requests) > 1 and not stat:
          gLogger.notice('\n===================================')
        dbStatus = reqClient.getRequestStatus(requestID).get('Value', 'Unknown')
        if not stat:
          printRequest(request, status=dbStatus, full=full, verbose=verbose, terse=terse)

  if stat:
    gLogger.notice("Found %d requests with RequestID: %s" %
                   (len(requests), ','.join(requestID for requestID in requests)))

  if listJobs:
    gLogger.notice("List of %d jobs:\n" % len(jobIDList),
                   ','.join(str(jobID) for jobID in jobIDList))

  if status and okRequests:
    from DIRAC.Core.Utilities.List import breakListIntoChunks
    gLogger.notice('\nList of %d selected requests:' % len(okRequests))
    for reqs in breakListIntoChunks(okRequests, 100):
      gLogger.notice(','.join(reqs))


if __name__ == "__main__":
  main()


import json
# # from DIRAC
from DIRAC import S_OK, S_ERROR
# from CTADIRAC
from CTADIRAC.DataManagementSystem.private.JSONUtils import DMSEncoder


class ProvBase(object):

  def toJSON(self):
    """ Returns the JSON formated string that describes the agent """
    try:
      jsonStr = json.dumps(self, cls=DMSEncoder)
      return S_OK(jsonStr)
    except Exception as e:
      return S_ERROR(str(e))

  def _jsonData(self, attrNames):
    """ Returns the JSON formated string that describes the agent """
    jsonData = {}
    for attrName in attrNames:
      # id might not be set since it is managed by SQLAlchemy
      if not hasattr(self, attrName):
        continue
      value = getattr(self, attrName)
      jsonData[attrName] = value

    return jsonData


class Activity(ProvBase):

  def __init__(self, id=None, name=None, startTime=None,
               endTime=None, comment=None, activityDescription_key=None):

    self.id = id
    self.name = name
    self.startTime = startTime
    self.endTime = endTime
    self.comment = comment
    self.activityDescription_key = activityDescription_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['id', 'name', 'startTime', 'endTime', 'comment', 'activityDescription_key']
    return self._jsonData(attrNames)


class Entity(ProvBase):

  def __init__(self, id=None, classType=None, name=None,
               location=None, generatedAtTime=None,
               invalidatedAtTime=None, comment=None, entityDescription_key=None):

    self.id = id
    self.classType = classType
    self.name = name
    self.location = location
    self.generatedAtTime = generatedAtTime
    self.invalidatedAtTime = invalidatedAtTime
    self.comment = comment
    self.entityDescription_key = entityDescription_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['id', 'classType', 'name', 'location', 'generatedAtTime', 'invalidatedAtTime', 'comment',
                 'entityDescription_key']
    return self._jsonData(attrNames)


class ValueEntity(Entity):

  def __init__(self, id=None, classType=None, name=None, location=None, generatedAtTime=None,
               invalidatedAtTime=None, comment=None, entityDescription_key=None, value=None):

    Entity.__init__(self, id=id, name=name, location=location, generatedAtTime=generatedAtTime,
                    invalidatedAtTime=invalidatedAtTime, comment=comment,
                    entityDescription_key=entityDescription_key, classType=classType)
    self.value = value


class DatasetEntity(Entity):

  def __init__(self, id=None, classType=None, name=None, location=None,
               generatedAtTime=None, invalidatedAtTime=None, comment=None, entityDescription_key=None,
               ctadirac_guid=None):

    Entity.__init__(self, id, classType, name, location, generatedAtTime,
                    invalidatedAtTime, comment, entityDescription_key)
    self.ctadirac_guid = ctadirac_guid

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['id', 'classType', 'name', 'location', 'generatedAtTime', 'invalidatedAtTime', 'comment',
                 'entityDescription_key', 'value']
    return self._jsonData(attrNames)


class Used(ProvBase):

  def __init__(self, role=None, time=None,
               activity_key=None, entity_key=None, usageDescription_key=None):

    self.role = role
    self.time = time
    self.activity_key = activity_key
    self.entity_key = entity_key
    self.usageDescription_key = usageDescription_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['role', 'time', 'activity_key', 'entity_key', 'usageDescription_key']
    return self._jsonData(attrNames)


class WasGeneratedBy(ProvBase):

  def __init__(self, id=None, role=None, activity_key=None,
               entity_key=None, generationDescription_key=None):

    self.role = role
    self.activity_key = activity_key
    self.entity_key = entity_key
    self.generationDescription_key = generationDescription_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['role', 'activity_key', 'entity_key', 'generationDescription_key']
    return self._jsonData(attrNames)


class Agent(ProvBase):

  def __init__(self, id=None, name=None, type=None, email=None,
               comment=None, affiliation=None, phone=None,
               address=None, url=None):

    self.id = id
    self.name = name
    self.type = type
    self.email = email
    self.comment = comment
    self.affiliation = affiliation
    self.phone = phone
    self.address = address
    self.url = url

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ["id", "name", "type", "email", "comment", "affiliation", "phone", "address", "url"]
    return self._jsonData(attrNames)


class WasAttributedTo(ProvBase):

  def __init__(self, role=None, entity_key=None, agent_key=None):

    self.role = role
    self.entity_key = entity_key
    self.agent_key = agent_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['role', 'entity_key', 'agent_key']
    return self._jsonData(attrNames)


class WasAssociatedWith(ProvBase):

  def __init__(self, role=None, activity_key=None, agent_key=None):

    self.role = role
    self.activity_key = activity_key
    self.agent_key = agent_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['role', 'activity_key', 'agent_key']
    return self._jsonData(attrNames)


class ActivityDescription(ProvBase):

  def __init__(self, name=None, version=None, description=None,
               type=None, subtype=None, doculink=None):
    self.name = name
    self.version = version
    self.description = description
    self.type = type
    self.subtype = subtype
    self.doculink = doculink

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['name', 'type', 'subtype', 'version', 'doculink', 'description']
    return self._jsonData(attrNames)


class EntityDescription(ProvBase):

  def __init__(self, name=None, type=None, description=None, doculink=None, classType=None):

    self.name = name
    self.type = type
    self.description = description
    self.doculink = doculink
    self.classType = classType

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['id', 'name', 'type', 'description', 'doculink', 'classType']
    return self._jsonData(attrNames)


class DatasetDescription(EntityDescription):

  def __init__(self, name=None, type=None, description=None, doculink=None, classType=None,
               contentType=None):

    EntityDescription.__init__(self, name=name, type=type, description=description,
                               doculink=doculink, classType=classType)
    self.contentType = contentType

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['name', 'type', 'description', 'doculink', 'contentType']
    return self._jsonData(attrNames)


class ValueDescription(EntityDescription):

  def __init__(self, name=None, type=None, description=None, doculink=None, classType=None,
               valueType=None, unit=None, ucd=None, utype=None):
    EntityDescription.__init__(self, name=name, type=type, description=description,
                               doculink=doculink, classType=classType)
    self.valueType = valueType
    self.unit = unit
    self.ucd = ucd
    self.utype = utype

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['name', 'type', 'description', 'doculink', 'valueType', 'unit', 'ucd', 'utype']
    return self._jsonData(attrNames)


class UsageDescription(ProvBase):

  def __init__(self, role=None, description=None, type=None, multiplicity=1,
               activityDescription_key=None, entityDescription_key=None):

    self.role = role
    self.description = description
    self.type = type
    self.multiplicity = multiplicity
    self.activityDescription_key = activityDescription_key
    self.entityDescription_key = entityDescription_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['role', 'description', 'type', 'multiplicity', 'activityDescription_key', 'entityDescription_key']
    return self._jsonData(attrNames)


class GenerationDescription(ProvBase):

  def __init__(self, role=None, description=None, type=None, multiplicity=1,
               activityDescription_key=None, entityDescription_key=None):

    self.role = role
    self.description = description
    self.type = type
    self.multiplicity = multiplicity
    self.activityDescription_key = activityDescription_key
    self.entityDescription_key = entityDescription_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['role', 'description', 'type', 'multiplicity', 'activityDescription_key', 'entityDescription_key']
    return self._jsonData(attrNames)


class WasConfiguredBy(ProvBase):

  def __init__(self, id=None, artefactType='Parameter', activity_key=None, parameter_key=None, configFile_key=None):

    self.id = id
    self.artefactType = artefactType
    self.activity_key = activity_key
    self.parameter_key = parameter_key
    self.configFile_key = configFile_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['id', 'artefactType', 'activity_key', 'parameter_key', 'configFile_key']
    return self._jsonData(attrNames)


class Parameter(ProvBase):

  def __init__(self, id=None, value=None, name=None, parameterDescription_key=None):

    self.id = id
    self.value = value
    self.name = name
    self.parameterDescription_key = parameterDescription_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['id', 'value', 'name', 'parameterDescription_key']
    return self._jsonData(attrNames)


class ConfigFile(ProvBase):

  def __init__(self, name=None, location=None, comment=None, configFileDescription_key=None):

    self.name = name
    self.location = location
    self.comment = comment
    self.configFileDescription_key = configFileDescription_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['name', 'location', 'comment', 'configFileDescription_key']
    return self._jsonData(attrNames)


class ParameterDescription(ProvBase):

  def __init__(self, name=None, valueType=None, description=None, unit=None, ucd=None,
               utype=None, min=None, max=None, default=None, options=None, activityDescription_key=None):

    self.name = name
    self.valueType = valueType
    self.description = description
    self.unit = unit
    self.ucd = ucd
    self.utype = utype
    self.min = min
    self.max = max
    self.default = default
    self.options = options
    self.activityDescription_key = activityDescription_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['name', 'valueType', 'description', 'unit', 'ucd', 'utype', 'min', 'max', 'default', 'options',
                 'activityDescription_key']
    return self._jsonData(attrNames)


class ConfigFileDescription(ProvBase):

  def __init__(self, id=None, name=None, contentType=None, description=None, activityDescription_key=None):

    self.id = id
    self.name = name
    self.contentType = contentType
    self.description = description
    self.activityDescription_key = activityDescription_key

  def _getJSONData(self):
    """ Returns the data that have to be serialized by JSON """

    attrNames = ['name', 'contentType', 'description', 'activityDescription_key']
    return self._jsonData(attrNames)



"""
  Wrapper on the Job class to handle a simple ctapipe jobs DL0->DL1->DL2
  following the early pipeline developments by T. Michael
  https://github.com/tino-michael/tino_cta

@authors: J. Bregeon, L. Arrabito, D. Landriu, J. Lefaucheur
            April 2018
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
import yaml

# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from CTADIRAC.Core.Utilities.tool_box import DATA_LEVEL_METADATA_ID


class SimpleCtapipeJob(Job):
  """ Job extension class for ctapipe experimental analysis scripts
  in early developments at CEA Saclay DL0->DL2
  Takes care of fine tuning the software installation,
                classify and reconstruction,
                merging
  """

  def __init__(self, cpuTime=36000):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__(self)
    self.setCPUTime(cpuTime)
    # defaults
    self.setName('ctapipe')
    self.package = 'ctapipe'
    self.program_category = 'calibimgreco'
    self.version = 'v0.5.3'
    self.configuration_id = 1  # To be set according to the input dataset
    # ctapipe params
    self.max_events = '10000000000'
    self.cam_ids = "LSTCam NectarCam"
    ########################################################################
    # data management params
    self.output_type = 'DL1'
    self.basepath = '/vo.cta.in2p3.fr/MC/PROD3/'
    self.metadata = collections.OrderedDict()
    self.filemetadata = {}
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])

  def loadConfig(self, config_file):
    """ Load the configuration file using yaml return the configuration dict

    Parameters:
    config_file -- configuration file for ctapipe
    """
    with open(config_file, 'rt') as stream:
      cfg = yaml.load(stream)

    return cfg

  def setConfig(self, config_file):
    """ Set configuration for ctapipe as in the user configuration file

    Parameters:
    config_file -- configuration file for ctapipe
    """
    # Read configuration file
    cfg = self.loadConfig(config_file)

    # Read General section
    self.ana_cfg = cfg['General']['config_file']
    self.modes = cfg['General']['modes']  # One mode now
    self.particle = cfg['General']['particle']
    self.estimate_energy = cfg['General']['estimate_energy']
    self.force_tailcut_for_extended_cleaning = cfg['General']['force_tailcut_for_extended_cleaning']

    if self.force_tailcut_for_extended_cleaning is True:
      self.force_modes = [mode.replace('wave', 'tail') for mode in self.modes]
    else:
      self.force_modes = self.modes

    # Read EnergyRegressor, GammaHadronClassifier, Performance sections
    self.energyRegressor = cfg['EnergyRegressor']
    self.gammaHadronClassifier = cfg['GammaHadronClassifier']
    self.performance = cfg['Performance']

  def getInputfile(self):
    """ Select the input file with the list of LFNs according to the configuration
          and return the selected input file
    """
    input_file = {}
    if self.estimate_energy is False and self.output_type == 'DL1':
      input_file['gamma'] = self.energyRegressor['gamma_list']
    elif self.estimate_energy is True and self.output_type == 'DL1':
      input_file['gamma'] = self.gammaHadronClassifier['gamma_list']
      input_file['proton'] = self.gammaHadronClassifier['proton_list']
    elif self.output_type == 'DL2':
      input_file['gamma'] = self.performance['gamma_list']
      input_file['proton'] = self.performance['proton_list']
      input_file['electron'] = self.performance['electron_list']
    return input_file[self.particle]

  def setCtapipeMD(self, simtelMD):
    """ Set ctapipe meta data

    Parameters:
    simtelMD -- metadata dictionary as for simtel
    """
    # set ctapipe directory meta data
    self.metadata['array_layout'] = simtelMD['array_layout']
    self.metadata['site'] = simtelMD['site']
    self.metadata['particle'] = simtelMD['particle']
    try:
      phiP = simtelMD['phiP']['=']
    except BaseException:
      phiP = simtelMD['phiP']
    self.metadata['phiP'] = phiP
    try:
      thetaP = simtelMD['thetaP']['=']
    except BaseException:
      thetaP = simtelMD['thetaP']
    self.metadata['thetaP'] = thetaP
    self.metadata[self.program_category + '_prog'] = self.package
    self.metadata[self.program_category + '_prog_version'] = self.version
    self.metadata['data_level'] = DATA_LEVEL_METADATA_ID[self.output_type]
    self.metadata['configuration_id'] = self.configuration_id

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called
    """

    # step 1 -- to be removed -- debug only
    iStep = 1
    if debug:
      step = self.setExecutable('/bin/ls -alhtr',
                                logFile='LS_Init_Log.txt')
      step['Value']['name'] = 'Step%i_LS_Init' % iStep
      step['Value']['descr_short'] = 'list files in working directory'
      iStep += 1

    # step 2
    step = self.setExecutable('cta-prod3-setupsw',
                              arguments='%s %s %s' %
                              (self.package, self.version, self.program_category),
                              logFile='SetupSoftware_Log.txt')
    step['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    step['Value']['descr_short'] = 'Setup software'
    iStep += 1

    # step 2bis
    # arguments are nbFiles=0 (not used) and fileSize=100kB
    step = self.setExecutable('cta-prod-verifysteps',
                              arguments='analysisinputs 0 100',
                              logFile='Verify_CtapipeInputs_Log.txt')
    step['Value']['name'] = 'Step%i_VerifyPipeInputs' % iStep
    step['Value']['descr_short'] = 'Verify ctapipe Inputs'
    iStep += 1

    # step 3: run ctapipe write_dl1 or write_dl2
    for mode in self.force_modes:
      if self.output_type == 'DL1':
        ctapipe_exe = './dirac_ctapipe_dl1'
        ctapipe_args = "--config_file %s --estimate_energy %s --max_events %s --mode %s --cam_ids '%s'" % \
            (self.ana_cfg, self.estimate_energy, self.max_events, mode, self.cam_ids)
      elif self.output_type == 'DL2':
        ctapipe_exe = './dirac_ctapipe_dl2'
        ctapipe_args = "--config_file %s --max_events %s --mode %s --force_tailcut_for_extended_cleaning %s --cam_ids '%s'" % \
            (self.ana_cfg, self.max_events, mode, self.force_tailcut_for_extended_cleaning, self.cam_ids)

      step = self.setExecutable(ctapipe_exe, arguments=ctapipe_args, logFile='Ctapipe_Log.txt')
      step['Value']['name'] = 'Step%i_Ctapipe' % iStep
      step['Value']['descr_short'] = 'Run Ctapipe'
      iStep += 1

      # step 4: run merge tables (if only 1 file is found the script exits gracefully)
      ctapipe_exe = './dirac_ctapipe_merge_tables'
      ctapipe_args = "--mode %s" % mode
      step = self.setExecutable(ctapipe_exe, arguments=ctapipe_args, logFile='MergeTables_Log.txt')
      step['Value']['name'] = 'Step%i_MergeTables' % iStep
      step['Value']['descr_short'] = 'Run Merge tables'
      iStep += 1

    # step 4
    # ## the order of the metadata dictionary is important, since it's used to build the directory structure
    mdjson = json.dumps(self.metadata)
    metadatafield = {'array_layout': 'VARCHAR(128)',
                     'site': 'VARCHAR(128)',
                     'particle': 'VARCHAR(128)', 'phiP': 'float',
                     'thetaP': 'float',
                     self.program_category + '_prog': 'VARCHAR(128)',
                     self.program_category + '_prog_version': 'VARCHAR(128)',
                     'data_level': 'int', 'configuration_id': 'int'}
    mdfieldjson = json.dumps(metadatafield)

    # register Data
    outputpattern = './*.h5'
    file_md_json = json.dumps(self.filemetadata)
    scripts = '../CTADIRAC/Core/scripts'
    step = self.setExecutable(scripts + '/cta-analysis-managedata.py',
                              arguments="'%s' '%s' '%s' %s '%s' %s %s '%s'" %
                              (mdjson, mdfieldjson, file_md_json, self.basepath,
                               outputpattern, self.package, self.program_category,
                               self.catalogs),
                              logFile='DataManagement_Log.txt')
    step['Value']['name'] = 'Step%i_DataManagement' % iStep
    step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    iStep += 1

    # step 5 -- to be removed -- debug only
    if debug:
      step = self.setExecutable('/bin/ls -alhtr ./ ',
                                logFile='LS_End_Log.txt')
      step['Value']['name'] = 'Step%i_LS_End' % iStep
      step['Value']['descr_short'] = 'list files in working and Data directory'
      iStep += 1



#!/usr/bin/env python
""" Simple data management script for PROD3 MC
    create DFC MetaData structure
    put and register files in DFC
"""

__RCSID__ = "$Id$"

# DIRAC imports
import DIRAC
from DIRAC.Core.Base import Script
 
Script.setUsageMessage( '\n'.join( [ __doc__.split( '\n' )[1],
                                     'Usage:',
                                     '  %s one two' % Script.scriptName,
                                     'Arguments:',
                                     '  one: one',
                                     '\ne.g: %s ?' % Script.scriptName
                                     ] ) )

Script.parseCommandLine()

# Specific DIRAC imports
from CTADIRAC.Core.Workflow.Modules.ProdDataManager import ProdDataManager

####################################################
def main():
    """ simple wrapper to remove PROD3 local files
    
    Keyword arguments:
    args -- a list of arguments in order []
    """
    args = Script.getPositionalArgs()
    datadir = args[0]
    pattern = args[1]
    catalogs = ['DIRACFileCatalog']
    
    prod3dm = ProdDataManager( catalogs )
    # ## Remove local files
    res = prod3dm.cleanLocalFiles( datadir, pattern )
    if not res['OK']:
      return res

    DIRAC.exit()

####################################################
if __name__ == '__main__':
  main()


#!/usr/bin/env python

"""
Remove files downloaded as job InputData having a Bad checksum
"""

__RCSID__ = "$Id$"

# Generic imports
import os

# DIRAC imports
import DIRAC
from DIRAC.Core.Base import Script
Script.parseCommandLine()

# Specific DIRAC imports
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
from DIRAC.Interfaces.API.Dirac import Dirac

def main():
  if 'JOBID' in os.environ:
    jobID = os.environ['JOBID']
    dirac = Dirac()
    res = dirac.getJobJDL(jobID)
    if 'InputData' in res['Value']:
      lfn = res['Value']['InputData']

  dm = DataManager(['DIRACFileCatalog'])
  DIRAC.gLogger.notice('Check file: %s' % lfn)

  res = dm.getFile(lfn)

  if lfn in res['Value']['Failed']:
    if 'Bad checksum' in res['Value']['Failed'][lfn]:
      DIRAC.gLogger.notice('Bad checksum. Removing: %s' % lfn)
      res = dm.removeFile(lfn)
      if res['OK']:
        DIRAC.gLogger.notice('Successfully removed: %s' % lfn)
      else:
        DIRAC.gLogger.error(res['Message'])
        DIRAC.exit(-1)
  else:
    DIRAC.gLogger.notice('Good checksum for: %s' % lfn)
    DIRAC.exit()

####################################################
if __name__ == '__main__':
  main()


"""
  Simple Wrapper on the Job class to handle Prod3 MC
  piping corsika into simtels
  created for Prod3 La Palma 3 production
			JB,LA Decembre 2016
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job


class Prod3MCPipeJob(Job):
  """ Job extension class for Prod3 MC simulations,
      takes care of running corsika piped into simtel
      one output file is created
  """

  def __init__(self, cpuTime=259200):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__(self)
    self.setCPUTime(cpuTime)
    # defaults
    self.setName('Prod3MC_Generation')
    self.package = 'corsika_simhessarray'
    self.version = '2016-12-02'
    self.nShower = 100
    self.start_run_number = '0'
    self.run_number = '10'
    self.cta_site = 'LaPalma'
    self.cta_site_tag = '3'
    self.array_layout = 'LaPalma3'
    self.particle = 'gamma'
    self.pointing_dir = 'South'
    self.zenith_angle = 20.
    self.inputpath = 'Data/sim_telarray/cta-prod3-%s/0.0deg' % self.array_layout.lower()
    self.basepath = '/vo.cta.in2p3.fr/MC/PROD3/'
    self.no_sct = True
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])

  def setPackage(self, package):
    """ Set package name : e.g. 'corsika_simhessarray'

    Parameters:
    package -- corsika_simhessarray
    """
    self.package = package

  def setVersion(self, version):
    """ Set software version number : e.g. 2016-12-02

    Parameters:
    version -- corsika+simtel package version number
    """
    self.version = version

  def setNShower(self, nshow):
    """ Set the number of corsika showers,
        5 is enough for testing
        20000 in production usually.

    Parameters:
    nshow -- number of corsika primary showers to generate
    """
    self.nShower = nshow

  def setRunNumber(self, runNb):
    """ Set the corsika run number, passed as a string
        because may be a TS parameter

    Parameters:
    runNb -- run number as a string, used as a corsika seed
    """
    self.run_number = runNb

  def setStartRunNumber(self, startrunNb):
    """ Set the corsika start run number (to be added to the run_number), passed as a string
        because may be a TS parameter

    Parameters:
    startrunNb -- to be added to the run number
    """
    self.start_run_number = startrunNb

  def setArrayLayout(self, layout):
    """ Set the array layout type

    Parameters:
    layout -- a string for the array layout, hex or square
    """
    if layout in ['hex', 'square', 'LaPalma3']:
      DIRAC.gLogger.info('Set Simtel layout to: %s' % layout)
      self.array_layout = layout
    else:
      DIRAC.gLogger.error('Unknown layout: : %s' % layout)
      DIRAC.exit(-1)

  def setSite(self, site):
    """ Set the site to simulate

    Parameters:
    site -- a string for the site name (LaPalma)
    """
    self.cta_site = site

  def setParticle(self, particle):
    """ Set the corsika primary particle

    Parameters:
    particle -- a string for the particle type/name
    """
    if particle in ['gamma', 'gamma-diffuse', 'electron', 'proton', 'helium']:
      DIRAC.gLogger.info('Set Corsika particle to: %s' % particle)
      self.particle = particle
    else:
      DIRAC.gLogger.error('Corsika does not know particle type: %s' % particle)
      DIRAC.exit(-1)

  def setPointingDir(self, pointing):
    """ Set the pointing direction, North or South

    Parameters:
    pointing -- a string for the pointing direction
    """
    if pointing in ['North', 'South', 'East', 'West']:
      DIRAC.gLogger.info('Set Pointing dir to: %s' % pointing)
      self.pointing_dir = pointing
    else:
      DIRAC.gLogger.error('Unknown pointing direction: %s' % pointing)
      DIRAC.exit(-1)

  def setZenithAngle(self, zenith):
    """ Set the pointing direction, North or South

    Parameters:
    zenith -- a string for the zenith angle
    """
    self.zenith_angle = zenith

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """
    # step 1 - debug only
    iStep = 1
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_Init' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory'
      iStep += 1

    # step 2
    swStep = self.setExecutable('cta-prod3-setupsw',
                                arguments='%s %s' % (self.package, self.version),
                                logFile='SetupSoftware_Log.txt')
    swStep['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    swStep['Value']['descr_short'] = 'Setup software'
    iStep += 1

    # step 3
    # no_sct option works only with version 2016-12-20b
    if self.no_sct:
      prod_script = './dirac_prod3_lapalma3_noSCT'
    else:
      prod_script = './dirac_prod3_lapalma3'
    csStep = self.setExecutable(prod_script,
                                arguments='--start_run %s --run %s %s %s %s %s' %
                                (self.start_run_number, self.run_number,
                                 self.cta_site,
                                 self.particle, self.pointing_dir, self.zenith_angle),
                                logFile='CorsikaSimtel_Log.txt')
    csStep['Value']['name'] = 'Step%i_CorsikaSimtel' % iStep
    csStep['Value']['descr_short'] = 'Run Corsika piped into simtel'
    iStep += 1

    # step 4 verify merged data
    mgvStep = self.setExecutable('cta-prod-verifysteps',
                                 arguments="generic 1 1000 '%s/Data/*.simtel.gz'" % self.inputpath,
                                 logFile='Verify_Simtel_Log.txt')
    mgvStep['Value']['name'] = 'Step%i_VerifySimtel' % iStep
    mgvStep['Value']['descr_short'] = 'Verify simtel files'
    iStep += 1

    # step 5 - debug only
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr Data/sim_telarray/*/*/*', logFile='LS_End_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_End' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory and sub-directory'
      iStep += 1

    # step 6
    # ## the order of the metadata dictionary is important, since it's used to build the directory structure
    metadata = collections.OrderedDict()
    metadata['array_layout'] = self.array_layout
    metadata['site'] = self.cta_site
    metadata['particle'] = self.particle
    if self.pointing_dir == 'North':
      metadata['phiP'] = 180
    if self.pointing_dir == 'South':
      metadata['phiP'] = 0
    metadata['thetaP'] = float(self.zenith_angle)
    metadata['tel_sim_prog'] = 'simtel'
    metadata['tel_sim_prog_version'] = self.version
    # new meta data introduced after Prod3b
    metadata['data_level'] = 0
    metadata['configuration_id'] = -1
    mdjson = json.dumps(metadata)

    metadatafield = {'array_layout': 'VARCHAR(128)',
                     'site': 'VARCHAR(128)',
                     'particle': 'VARCHAR(128)', 'phiP': 'float',
                     'thetaP': 'float',
                     'tel_sim_prog': 'VARCHAR(128)',
                     'tel_sim_prog_version': 'VARCHAR(128)',
                     'data_level': 'int', 'configuration_id': 'int'}

    mdfieldjson = json.dumps(metadatafield)

    filemetadata = {'runNumber': self.run_number}
    # 'sct': False}

    fmdjson = json.dumps(filemetadata)
    scripts = '../CTADIRAC/Core/scripts/'
    dmStep = self.setExecutable(scripts + 'cta-prod3-managedata.py',
                                arguments="'%s' '%s' '%s' %s %s %s '%s'" %
                                (mdjson, mdfieldjson, fmdjson, self.inputpath,
                                 self.basepath, self.start_run_number, self.catalogs),
                                logFile='DataManagement_Log.txt')
    dmStep['Value']['name'] = 'Step%i_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save files to SE and register them in DFC'
    iStep += 1

    # Number of showers is passed via an environment variable
    self.setExecutionEnv({'NSHOW': '%s' % self.nShower})



"""
  Wrapper on the Job class to handle Prod3 MC
  piping corsika into 3 simtels
  created for Prod3 NSB tests with reference base line
  for both Paranal
			JB,LA September 2017
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
import DIRAC
# Base class
from CTADIRAC.Interfaces.API.Prod3MCPipeBaselineJob import Prod3MCPipeBaselineJob


class Prod3MCPipeBaselineNSBJob(Prod3MCPipeBaselineJob):
  """ Job extension class for Prod3 MC NSB simulations,
      takes care of running corsika piped into simtel
      3 output files are created
      Most code is inherited from the Prod3MCPipeBaselineJob class
  """

  def __init__(self, cpuTime=259200):
    """ Constructor takes almosst everything from base class

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
#    super(Prod3MCPipeBaselineJob, self).__init__()
    Prod3MCPipeBaselineJob.__init__(self, cpuTime)
#    self.inputpath = 'Data/sim_telarray/cta-prod3-demo/0.0deg'
    self.N_output_files = 3

  def setupWorkflow(self, debug=False):
    """ Override the base class job workflow to adapt to NSB test simulations
        All parameters shall have been defined before that method is called.
    """
    # step 1 - debug only
    iStep = 1
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_Init' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory'
      iStep += 1

    # step 2
    swStep = self.setExecutable('cta-prod3-setupsw',
                                arguments='%s %s' % (self.package, self.version),
                                logFile='SetupSoftware_Log.txt')
    swStep['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    swStep['Value']['descr_short'] = 'Setup software'
    iStep += 1

    # step 3
    if self.cta_site == 'Paranal':
      prod_script = './dirac_prod3_paranal_baseline'
    elif self.cta_site == 'LaPalma':
      prod_script = './dirac_prod3_lapalma_baseline'
    else:
      DIRAC.gLogger.error('Unknown site: %s' % self.cta_site)
      DIRAC.gLogger.error('No shell script associated')
      DIRAC.exit(-1)

    csStep = self.setExecutable(prod_script,
                                arguments='--start_run %s --run %s %s %s %s %s' %
                                (self.start_run_number, self.run_number,
                                 self.cta_site,
                                 self.particle, self.pointing_dir, self.zenith_angle),
                                logFile='CorsikaSimtel_Log.txt')
    csStep['Value']['name'] = 'Step%i_CorsikaSimtel' % iStep
    csStep['Value']['descr_short'] = 'Run Corsika piped into simtel'
    iStep += 1

    # step 4 verify merged data
    mgvStep = self.setExecutable('ta-prod-verifysteps',
                                 arguments="generic %0d 1000 '%s/Data/*.simtel.gz'" %
                                 (self.N_output_files, self.inputpath),
                                 logFile='Verify_Simtel_Log.txt')
    mgvStep['Value']['name'] = 'Step%i_VerifySimtel' % iStep
    mgvStep['Value']['descr_short'] = 'Verify simtel files'
    iStep += 1

    # step 5 - debug only
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr Data/sim_telarray/*/*/*', logFile='LS_End_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_End' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory and sub-directory'
      iStep += 1

    # step 6
    # meta data
    # ## the order of the metadata dictionary is important, since it's used to build the directory structure
    metadata = collections.OrderedDict()
    metadata['array_layout'] = self.array_layout
    metadata['site'] = self.cta_site
    metadata['particle'] = self.particle
    if self.pointing_dir == 'North':
      metadata['phiP'] = 180
    if self.pointing_dir == 'South':
      metadata['phiP'] = 0
    metadata['thetaP'] = float(self.zenith_angle)
    metadata[self.program_category + '_prog'] = 'simtel'
    metadata[self.program_category + '_prog_version'] = self.version
    metadata['data_level'] = self.output_data_level
    metadata['configuration_id'] = self.configuration_id
    mdjson = json.dumps(metadata)

    # meta data field
    metadatafield = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                     'particle': 'VARCHAR(128)', 'phiP': 'float',
                     'thetaP': 'float',
                     self.program_category + '_prog': 'VARCHAR(128)',
                     self.program_category + '_prog_version': 'VARCHAR(128)',
                     'data_level': 'int', 'configuration_id': 'int'}
    mdfieldjson = json.dumps(metadatafield)

    # register Data NSB 1x
    # file meta data
    filemetadata = {'runNumber': self.run_number, 'nsb': 1}
    file_md_json = json.dumps(filemetadata)
    outputpattern = './Data/sim_telarray/*/*/Data/*baseline.simtel.gz'
    if self.particle == 'gamma-diffuse':
      outputpattern = './Data/sim_telarray/*/*/Data/*baseline_cone10.simtel.gz'
    dmStep = self.setExecutable('cta-analysis-managedata',
                                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s'" %
                                (mdjson, mdfieldjson, file_md_json, self.basepath,
                                 outputpattern, self.package, self.program_category, self.catalogs),
                                logFile='DataManagement_NSB1x_Log.txt')
    dmStep['Value']['name'] = 'Step%i_DataManagement_1' % iStep
    dmStep['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    iStep += 1
    # log file
    outputpattern = './Data/sim_telarray/*/*/Log/*baseline.log.gz'
    if self.particle == 'gamma-diffuse':
      outputpattern = './Data/sim_telarray/*/*/Log/*baseline_cone10.log.gz'
    dmStep = self.setExecutable('cta-analysis-managedata',
                                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                (mdjson, mdfieldjson, file_md_json, self.basepath,
                                 outputpattern, self.package, self.program_category, self.catalogs),
                                logFile='Log_DataManagement_NSB1x_Log.txt')
    dmStep['Value']['name'] = 'Step%i_Log_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save log files to SE and register them in DFC'
    iStep += 1
    # histogram
    outputpattern = './Data/sim_telarray/*/*/Histograms/*baseline.hdata.gz'
    if self.particle == 'gamma-diffuse':
      outputpattern = './Data/sim_telarray/*/*/Histograms/*baseline_cone10.hdata.gz'
    dmStep = self.setExecutable('cta-analysis-managedata',
                                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Histograms" %
                                (mdjson, mdfieldjson, file_md_json, self.basepath,
                                 outputpattern, self.package, self.program_category, self.catalogs),
                                logFile='Histo_DataManagement_NSB1x_Log.txt')
    dmStep['Value']['name'] = 'Step%i_Histo_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save hitograms files to SE and register them in DFC'
    iStep += 1

    # register Data NSB 5x
    # file meta data
    filemetadata = {'runNumber': self.run_number, 'nsb': 5}
    file_md_json = json.dumps(filemetadata)
    outputpattern = './Data/sim_telarray/*/*/Data/*baseline-5x*.simtel.gz'
    dmStep = self.setExecutable('cta-analysis-managedata',
                                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s'" %
                                (mdjson, mdfieldjson, file_md_json, self.basepath,
                                 outputpattern, self.package, self.program_category, self.catalogs),
                                logFile='DataManagement_NSB5x_Log.txt')
    dmStep['Value']['name'] = 'Step%i_DataManagement_1' % iStep
    dmStep['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    iStep += 1
    # log file
    outputpattern = './Data/sim_telarray/*/*/Log/*baseline-5x*.log.gz'
    dmStep = self.setExecutable('cta-analysis-managedata',
                                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                (mdjson, mdfieldjson, file_md_json, self.basepath,
                                 outputpattern, self.package, self.program_category, self.catalogs),
                                logFile='Log_DataManagement_NSB5x_Log.txt')
    dmStep['Value']['name'] = 'Step%i_Log_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save log files to SE and register them in DFC'
    iStep += 1
    # histogram
    outputpattern = './Data/sim_telarray/*/*/Histograms/*baseline-5x*.hdata.gz'
    dmStep = self.setExecutable('cta-analysis-managedata',
                                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Histograms" %
                                (mdjson, mdfieldjson, file_md_json, self.basepath,
                                 outputpattern, self.package, self.program_category, self.catalogs),
                                logFile='Histo_DataManagement_NSB5x_Log.txt')
    dmStep['Value']['name'] = 'Step%i_Histo_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save histogram files to SE and register \
                                      them in DFC'
    iStep += 1

    # register Data NSB 30x for Paranal only
    if self.cta_site == 'Paranal':
      # file meta data
      filemetadata = {'runNumber': self.run_number, 'nsb': 30}
      file_md_json = json.dumps(filemetadata)
      outputpattern = './Data/sim_telarray/*/*/Data/*baseline-30x*.simtel.gz'
      dmStep = self.setExecutable('cta-analysis-managedata',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s'" %
                                  (mdjson, mdfieldjson, file_md_json, self.basepath,
                                   outputpattern, self.package, self.program_category, self.catalogs),
                                  logFile='DataManagement_NSB30x_Log.txt')
      dmStep['Value']['name'] = 'Step%i_DataManagement_1' % iStep
      dmStep['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
      iStep += 1
      # log file
      outputpattern = './Data/sim_telarray/*/*/Log/*baseline-30x*.log.gz'
      dmStep = self.setExecutable('cta-analysis-managedata',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                  (mdjson, mdfieldjson, file_md_json, self.basepath,
                                   outputpattern, self.package, self.program_category, self.catalogs),
                                  logFile='Log_DataManagement_NSB30x_Log.txt')
      dmStep['Value']['name'] = 'Step%i_Log_DataManagement' % iStep
      dmStep['Value']['descr_short'] = 'Save log files to SE and register them in DFC'
      iStep += 1
      # histogram
      outputpattern = './Data/sim_telarray/*/*/Histograms/*baseline-30x*.hdata.gz'
      dmStep = self.setExecutable('cta-analysis-managedata',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Histograms" %
                                  (mdjson, mdfieldjson, file_md_json, self.basepath,
                                   outputpattern, self.package, self.program_category, self.catalogs),
                                  logFile='Histo_DataManagement_NSB30x_Log.txt')
      dmStep['Value']['name'] = 'Step%i_Histo_DataManagement' % iStep
      dmStep['Value']['descr_short'] = 'Save histograms files to SE and\
                                          register them in DFC'
      iStep += 1

    # Number of showers is passed via an environment variable
    self.setExecutionEnv({'NSHOW': '%s' % self.nShower})



#!/usr/bin/env python

"""
  Expects an input file with a list of lfns with the corresponding transID
  Set transformation files to a new status

  Usage:
    cta-transformation-set-file-status <ascii file with a list of (transID, lfn)> <newStatus>
"""

__RCSID__ = "$Id$"

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

def read_inputs(file_path):
  content = open(file_path, 'rt').readlines()
  resList = []
  for line in content:
    transID = line.strip().split(" ")[0]
    lfn = line.strip().split(" ")[1]
    if line != "\n":
      resList.append((transID, lfn))
  return resList

@Script()
def main():
  _, argss = Script.parseCommandLine(ignoreErrors=True)
  if len(argss) != 2:
    Script.showHelp()
      
  infile = argss[0]
  newFileStatus = argss[1]
  resList = read_inputs(infile)

  tc = TransformationClient()

  for (transID, lfn) in resList:
    res = tc.setFileStatusForTransformation(transID, newLFNsStatus=newFileStatus, lfns=[lfn])
    if not res['OK']:
      gLogger.error(res['Message'])
      DIRAC.exit(-1)

if __name__ == "__main__":
  main()


#!/usr/bin/env python

__RCSID__ = "$Id$"

# DIRAC imports
import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

Script.setUsageMessage("""
Treat problematic tasks for files with No replicas
Unregister files from the catalog and set transformation files to Processed

Usage:
   cta-prod-treat-problematic-transfers <ascii file with list of (transID lfn)>

Arguments:
  <ascii file with list of (transID lfn)> : input file with (transID lfn) to treat
""")

Script.parseCommandLine(ignoreErrors=True)
# Must comes after parseCommandLine
from DIRAC.Resources.Catalog.FileCatalog import FileCatalog
fc = FileCatalog()

def read_inputs(file_path):
  content = open(file_path, 'rt').readlines()
  resList = []
  for line in content:
    transID = line.strip().split(" ")[0]
    lfn = line.strip().split(" ")[1]
    if line != "\n":
      resList.append((transID, lfn))
  return resList

def unregisterFile(lfn):
  res = fc.removeFile(lfn)
  if res['OK']:
    if 'Failed' in res['Value']:
      if lfn in res['Value']['Failed']:
        gLogger.error(res['Value']['Failed'][lfn])
      elif lfn in res['Value']['Successful']:
        gLogger.notice("Successfully removed from the catalog", lfn)
      else:
        gLogger.error("Unexpected error result", res['Value'])
    else:
      gLogger.notice("Successfully removed from the catalog", lfn)
  else:
    gLogger.error("Failed to remove file from the catalog:", res['Message'])

  if not res['OK']:
    gLogger.error(res['Message'])

  return DIRAC.S_OK()

@Script()
def main():

  args = Script.getPositionalArgs()
  if len(args) != 1:
    Script.showHelp()

  infile = args[0]
  resList = read_inputs(infile)

  # Initialize the transDict with the transID keys
  transDict = {}
  for (transID, lfn) in resList:
    transDict.update({transID:[]})

  # Unregister file from the catalog and update the transDict with values
  for (transID, lfn) in resList:
    res = unregisterFile(lfn)
    if not res['OK']:
      gLogger.error(res['Message'])
      DIRAC.exit(-1)
    transDict[transID].append(lfn)

  tc = TransformationClient()

  for transID in transDict:
    res = tc.setFileStatusForTransformation(transID, newLFNsStatus='Processed', lfns=transDict[transID])
    if not res['OK']:
      gLogger.error(res['Message'])
      DIRAC.exit(-1)

if __name__ == '__main__':
  main()



#!/usr/bin/env python
""" Prod3 MC Script to setup software
"""

__RCSID__ = "$Id$"

# DIRAC imports
import DIRAC
from DIRAC.Core.Base import Script


Script.setUsageMessage( '\n'.join( [ __doc__.split( '\n' )[1],
                                     'Usage:',
                                     '  %s package version [program_category] [arch]' % Script.scriptName,
                                     'Arguments:',
                                     '  package: corsika_simhessarray',
                                     '  version: 2015-06-02',
                                     '\ne.g: %s corsika_simhessarray 2015-06-02'% Script.scriptName,
                                     ] ) )

Script.parseCommandLine()

# Specific DIRAC imports
from CTADIRAC.Core.Utilities.Prod3SoftwareManager import Prod3SoftwareManager

def main():
  """ setup a given software package
        to be used in main
    Keyword arguments:
    args -- a list of arguments in order [package, version, arch]
  """
  # check number of arguments
  args = Script.getPositionalArgs()
  if len( args ) < 2:
    Script.showHelp()
    return DIRAC.S_ERROR( 'Wrong number of arguments' )

  # get arguments
  package = args[0]
  version = args[1]
  program_category = 'simulations' 
  if len( args ) >= 3:
    program_category = args[2]
  arch = "sl6-gcc44"
  if len( args ) == 4:
    arch = args[3]

  soft_category = {package:program_category}
  prod3swm = Prod3SoftwareManager( soft_category )
  # check if and where Package is installed
  res = prod3swm.checkSoftwarePackage( package, version, arch )
  if not res['OK']:
    res = prod3swm.installSoftwarePackage( package, version, arch )
    if not res['OK']:
      return res
    else:
      package_dir = res['Value']
      prod3swm.dumpSetupScriptPath( package_dir )
      return DIRAC.S_OK()

  # # dump the SetupScriptPath to be sourced by DIRAC scripts
  # ## copy DIRAC scripts in the current directory
  package_dir = res['Value']
  prod3swm.dumpSetupScriptPath( package_dir )
  res = prod3swm.installDIRACScripts( package_dir )
  if not res['OK']:
    return res

  DIRAC.exit()

####################################################
if __name__ == '__main__':
  main()


"""
  Adapted job class to run tests in Prod3b conditions
  Wrapper on the Job class to handle Prod3 MC
  Paranal simulation with baseline layout, using zstd compression
  for simtel output files
			JB, March 2020
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
import DIRAC
# Base class
from CTADIRAC.Interfaces.API.Prod3MCPipeBaselineJob import Prod3MCPipeBaselineJob


class Prod3MCPipeTestJob(Prod3MCPipeBaselineJob):
  """ Job extension class for Prod3 MC NSB simulations,
      takes care of running corsika piped into simtel
      3 output files are created
      Most code is inherited from the Prod3MCPipeBaselineJob class
  """

  def __init__(self, cpuTime=259200):
    """ Constructor takes almosst everything from base class

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Prod3MCPipeBaselineJob.__init__(self, cpuTime)
    self.package = 'corsika_simtelarray'
    self.version = '2019-09-03'

  def setupWorkflow(self, debug=False):
    """ Override the base class job workflow to adapt to NSB test simulations
        All parameters shall have been defined before that method is called.
    """
    # step 1 - debug only
    iStep = 1
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_Init' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory'
      iStep += 1

      envStep = self.setExecutable('/bin/env', logFile='Env_Log.txt')
      envStep['Value']['name'] = 'Step%i_Env' % iStep
      envStep['Value']['descr_short'] = 'Dump environment'
      iStep += 1

    # step 2
    swStep = self.setExecutable('cta-prod-setup-software',
                                arguments='-p %s -v %s -a simulations -g gcc48_avx2' % (self.package, self.version),\
                                # arguments='-p %s -v %s'% (self.package, self.version),\
                                # swStep = self.setExecutable( 'cta-prod3-setupsw',
                                # arguments='%s %s simulations centos7-gcc48'% (self.package,
                                # self.version),\
                                logFile='SetupSoftware_Log.txt')
    swStep['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    swStep['Value']['descr_short'] = 'Setup software'
    iStep += 1

    # step 3
    if self.cta_site == 'Paranal':
      prod_script = './dirac_prod3_paranal_baseline'
    elif self.cta_site == 'LaPalma':
      prod_script = './dirac_prod3_lapalma_baseline'
    else:
      DIRAC.gLogger.error('Unknown site: %s' % self.cta_site)
      DIRAC.gLogger.error('No shell script associated')
      DIRAC.exit(-1)

    csStep = self.setExecutable(prod_script,
                                arguments='--start_run %s --run %s %s %s %s %s' %
                                (self.start_run_number, self.run_number,
                                 self.cta_site,
                                 self.particle, self.pointing_dir, self.zenith_angle),
                                logFile='CorsikaSimtel_Log.txt')
    csStep['Value']['name'] = 'Step%i_CorsikaSimtel' % iStep
    csStep['Value']['descr_short'] = 'Run Corsika piped into simtel'
    iStep += 1

    # step 4 verify merged data
    mgvStep = self.setExecutable('cta-prod-verifysteps',
                                 arguments="generic %0d 1000 '%s/Data/*.zst'" %
                                 (self.N_output_files, self.inputpath),
                                 logFile='Verify_Simtel_Log.txt')
    mgvStep['Value']['name'] = 'Step%i_VerifySimtel' % iStep
    mgvStep['Value']['descr_short'] = 'Verify simtel files'
    iStep += 1

    # step 5 - debug only
    if debug:
      lsStep = self.setExecutable('/bin/ls -Ralhtr', logFile='LS_End_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_End' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory and sub-directory'
      iStep += 1

    # step 6
    # meta data
    # ## the order of the metadata dictionary is important, since it's used to build the directory structure
    metadata = collections.OrderedDict()
    metadata['array_layout'] = self.array_layout
    metadata['site'] = self.cta_site
    metadata['particle'] = self.particle
    if self.pointing_dir == 'North':
      metadata['phiP'] = 180
    if self.pointing_dir == 'South':
      metadata['phiP'] = 0
    metadata['thetaP'] = float(self.zenith_angle)
    metadata[self.program_category + '_prog'] = 'simtel'
    metadata[self.program_category + '_prog_version'] = self.version
    metadata['data_level'] = self.output_data_level
    metadata['configuration_id'] = self.configuration_id
    mdjson = json.dumps(metadata)

    # meta data field
    metadatafield = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                     'particle': 'VARCHAR(128)', 'phiP': 'float',
                     'thetaP': 'float',
                     self.program_category + '_prog': 'VARCHAR(128)',
                     self.program_category + '_prog_version': 'VARCHAR(128)',
                     'data_level': 'int', 'configuration_id': 'int'}
    mdfieldjson = json.dumps(metadatafield)

    # register Data
    # file meta data
    #filemetadata = {'runNumber': self.run_number, 'nsb':1}
    filemetadata = {'runNumber': self.run_number}
    file_md_json = json.dumps(filemetadata)
    outputpattern = './Data/sim_telarray/*/*/Data/*baseline.simtel.zst'
    dmStep = self.setExecutable('../CTADIRAC/Core/scripts/cta-analysis-managedata.py',
                                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s'" %
                                (mdjson, mdfieldjson, file_md_json, self.basepath,
                                 outputpattern, self.package, self.program_category, self.catalogs),
                                logFile='Data_DataManagement_Log.txt')
    dmStep['Value']['name'] = 'Step%i_DataManagement_1' % iStep
    dmStep['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    iStep += 1
    # log file
    outputpattern = './Data/sim_telarray/*/*/Log/*baseline.log.gz'
    dmStep = self.setExecutable('../CTADIRAC/Core/scripts/cta-analysis-managedata.py',
                                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                (mdjson, mdfieldjson, file_md_json, self.basepath,
                                 outputpattern, self.package, self.program_category, self.catalogs),
                                logFile='Log_DataManagement_Log.txt')
    dmStep['Value']['name'] = 'Step%i_Log_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save log files to SE and register them in DFC'
    iStep += 1
    # histogram
    outputpattern = './Data/sim_telarray/*/*/Histograms/*baseline.hdata.gz'
    dmStep = self.setExecutable('../CTADIRAC/Core/scripts/cta-analysis-managedata.py',
                                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Histograms" %
                                (mdjson, mdfieldjson, file_md_json, self.basepath,
                                 outputpattern, self.package, self.program_category, self.catalogs),
                                logFile='Histo_DataManagement_Log.txt')
    dmStep['Value']['name'] = 'Step%i_Histo_DataManagement' % iStep
    dmStep['Value']['descr_short'] = 'Save hitograms files to SE and register them in DFC'
    iStep += 1

    # Number of showers is passed via an environment variable
    self.setExecutionEnv({'NSHOW': '%s' % self.nShower})



"""
  Job wrapper class to handle Prod4 MC
  Corsika simulations of the LST+MAGIC array
			JB Februrary 2019
"""

__RCSID__ = "$Id$"

# generic imports
import json

# DIRAC imports
import DIRAC
from CTADIRAC.Interfaces.API.Prod4CorsikaSSTJob import Prod4CorsikaSSTJob


class Prod4CorsikaLSTMagicJob(Prod4CorsikaSSTJob):
  """ Job extension class for Prod4 MC simulations,
    takes care of running corsika for the LST+MAGIC positions
    Inherits from Prod4CorsikaSSTJob
    @todo should have a Prod4CorsikaJob really
  """

  def __init__(self, cpu_time=129600):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Prod4CorsikaSSTJob.__init__(self)
    self.setCPUTime(cpu_time)
    self.version = '2018-11-07'
    self.configuration_id = 5
    self.cta_site = 'LaPalma'
    self.output_pattern = 'Data/corsika/run*/*corsika.zst'

  def set_meta_data(self):
    """ define the common meta data of the application
    """
    # The order of the metadata dictionary is important,
    # since it's used to build the directory structure
    self.metadata['array_layout'] = 'Baseline-LSTMagic'
    self.metadata['site'] = self.cta_site
    self.metadata['particle'] = self.particle
    # air shower simulation means North=0 and South=180
    if self.pointing_dir == 'North':
      self.metadata['phiP'] = 0
    if self.pointing_dir == 'South':
      self.metadata['phiP'] = 180
    self.metadata['thetaP'] = float(self.zenith_angle)
    self.metadata[self.program_category + '_prog'] = self.prog_name
    self.metadata[self.program_category + '_prog_version'] = self.version
    self.metadata['data_level'] = self.output_data_level
    self.metadata['configuration_id'] = self.configuration_id

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """
    # Step 1 - debug only
    i_step = 1

    # Step 2 - setup software
    sw_step = self.setExecutable('cta-prod3-setupsw',
                                 arguments='%s %s' % (self.package, self.version),
                                 logFile='SetupSoftware_Log.txt')
    sw_step['Value']['name'] = 'Step%i_SetupSoftware' % i_step
    sw_step['Value']['descr_short'] = 'Setup software'
    i_step += 1

    if debug:
      ls_step = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
      ls_step['Value']['name'] = 'Step%i_LS_Init' % i_step
      ls_step['Value']['descr_short'] = 'list files in working directory'
      i_step += 1

    # Step 3 - run corsika
    if self.cta_site == 'LaPalma':
      prod_script = './dirac_prod4_lst-magic_run'
    else:
      DIRAC.gLogger.error('Site not supported: %s' % self.cta_site)
      DIRAC.gLogger.error('No shell script associated')
      DIRAC.exit(-1)

    cs_step = self.setExecutable(prod_script,
                                 arguments='--without-multipipe --start_run %s \
                                               --run %s %s %s %s %s' %
                                 (self.start_run_number, self.run_number,
                                  self.cta_site, self.particle,
                                  self.pointing_dir, self.zenith_angle),
                                 logFile='Corsika_Log.txt')
    cs_step['Value']['name'] = 'Step%i_Corsika' % i_step
    cs_step['Value']['descr_short'] = 'Run Corsika only'
    i_step += 1

    # Step 4 - verify size of corsika output
    csv_step = self.setExecutable('cta-prod-verifysteps',
                                  arguments='generic %d %d %s' %
                                  (self.n_output_files, self.output_file_size,
                                   self.output_pattern),
                                  logFile='Verify_Corsika_Log.txt')
    csv_step['Value']['name'] = 'Step%i_VerifyCorsika' % i_step
    csv_step['Value']['descr_short'] = 'Verify the Corsika run'
    i_step += 1

    # Step 5 - define meta data, upload file on SE and register in catalogs
    self.set_meta_data()
    md_json = json.dumps(self.metadata)

    meta_data_field = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                       'particle': 'VARCHAR(128)',
                       'phiP': 'float', 'thetaP': 'float',
                       self.program_category + '_prog': 'VARCHAR(128)',
                       self.program_category + '_prog_version': 'VARCHAR(128)',
                       'data_level': 'int', 'configuration_id': 'int'}
    md_field_json = json.dumps(meta_data_field)

    # Upload and register data
    file_meta_data = {}
    file_md_json = json.dumps(file_meta_data)

    scripts = '../CTADIRAC/Core/scripts/'
    dm_step = self.setExecutable(scripts + 'cta-prod-managedata.py',
                                 arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                 (md_json, md_field_json, file_md_json,
                                  self.base_path, self.output_pattern, self.package,
                                  self.program_category, self.catalogs),
                                 logFile='DataManagement_Log.txt')
    dm_step['Value']['name'] = 'Step%s_DataManagement' % i_step
    dm_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    i_step += 1

    # Upload and register log file
    file_meta_data = {}
    file_md_json = json.dumps(file_meta_data)
    log_file_pattern = 'Data/corsika/run*/run*.log'
    scripts = '../CTADIRAC/Core/scripts/'
    log_step = self.setExecutable(scripts + 'cta-prod-managedata.py',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                  (md_json, md_field_json, file_md_json,
                                   self.base_path, log_file_pattern, self.package,
                                   self.program_category, self.catalogs),
                                  logFile='LogManagement_Log.txt')
    log_step['Value']['name'] = 'Step%s_LogManagement' % i_step
    log_step['Value']['descr_short'] = 'Save log to SE and register them in DFC'
    i_step += 1

    # Step 6 - debug only
    if debug:
      ls_step = self.setExecutable('/bin/ls -alhtr', logFile='LS_End_Log.txt')
      ls_step['Value']['name'] = 'Step%s_LSHOME_End' % i_step
      ls_step['Value']['descr_short'] = 'list files in Home directory'
      i_step += 1

    # Number of showers is passed via an environment variable
    self.setExecutionEnv({'NSHOW': '%s' % self.n_shower})
